#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService
# state        public
#----------------------------------------------------------------------------------------
# description  Service to connect to Google Ads API services
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService;

use base DE_EPAGES::GoogleSmartShopping::API::GoogleApiClient;
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( EPAGESJ_BASE_URI );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone );

use strict;

use constant CAMPAIGN_STATUS_STRINGS => {
    0 => 'PAUSED',
    1 => 'ENABLED'
};

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new GoogleAdsApiService.
#----------------------------------------------------------------------------------------
# return       GoogleAdsApiService | object | DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService
#========================================================================================
sub new {
    my $class = shift;
    my ( $hSettings ) = @_;

    my $self = $class->SUPER::new($hSettings);

    return $self;
}

#========================================================================================
# function     getGoogleAdsAccounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->getGoogleAdsAccounts();
#----------------------------------------------------------------------------------------
# description  Returns GoogleAdsAccounts array, or empty array if no Ads accounts found.
#----------------------------------------------------------------------------------------
# return       $aResponse | array with Google Ads Accounts | array.ref
#========================================================================================
sub getGoogleAdsAccounts {
    my $self = shift;

    my $Url = EPAGESJ_BASE_URI . 'accounts/ads/list';
    my $Headers = {'X-DeveloperToken' => $self->_getDeveloperToken()};

    my $aResponse;

    $aResponse =  $self->doGet($Url, $Headers);

    return $aResponse;
}

#========================================================================================
# function     createGoogleAdsSubAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleAdsSubAccount($Email, $Name);
#----------------------------------------------------------------------------------------
# description  Creates GoogleAds Sub Account
#----------------------------------------------------------------------------------------
# return       $hResponse | hash with Google Ads Account Id and name | hash.ref
#========================================================================================
sub createGoogleAdsSubAccount {
    my $self = shift;
    my ($Email, $Name) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = EPAGESJ_BASE_URI . "accounts/ads/$ManagerAccountId/sub-accounts";
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        name =>  $Name,
        currencyCode => $self->_getCurrencyCode(),
        timeZone => $self->_getTimeZone(),
        email=> $Email,
    };

    LogGoogleSmartShopping('createGoogleAdsAccount request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleAdsAccount response: ', {
        Response => $hResponse,
    });

    return $hResponse;

}

#========================================================================================
# function     createGoogleAdsLink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleAdsLink();
#----------------------------------------------------------------------------------------
# description  Creates Link between MerchantCenter Account and Ads Account
#========================================================================================
sub createGoogleAdsLink {
    my $self = shift;

    $self->loadGoogleSettings();

    my $hGoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'}->get([qw(
        GoogleAdsAccountID
        MerchantCenterAccountID
        MerchantCenterMultiClientAccountID
    )]);

    my $AdsAccountId = $hGoogleSmartShoppingConfig->{'GoogleAdsAccountID'};
    my $MerchantCenterAccountID = $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};
    my $MultiClientAccountID = $hGoogleSmartShoppingConfig->{'MerchantCenterMultiClientAccountID'} // $hGoogleSmartShoppingConfig->{'MerchantCenterAccountID'};

    my $Url = EPAGESJ_BASE_URI . "accounts/$MultiClientAccountID/sub-accounts/$MerchantCenterAccountID/ads-link";
    my $hHeaders = {};

    if ($MultiClientAccountID eq $self->_getEpagesMultiCLientAccountId()) {
        $hHeaders->{'AdminToken'} = $self->_createAdminToken();
    };

    my $hPayload = {
        adsAccountId =>  $AdsAccountId,
    };

    LogGoogleSmartShopping('createGoogleAdsLink request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPut($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleAdsLink response: ', {
        Response => $hResponse,
    });

    return;
}

#========================================================================================
# function     enableGoogleAdsLink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->enableGoogleAdsLink();
#----------------------------------------------------------------------------------------
# description  Enables Link between MerchantCenter Account and Ads Account
#               Only supports newly ePages created Accounts
#========================================================================================
sub enableGoogleAdsLink {
    my $self = shift;

    $self->loadGoogleSettings();

    my $AdsAccountId = $self->{'GoogleSmartShoppingConfig'}->get('GoogleAdsAccountID');
    my $AdsManagerAccountId = $self->_getAdsManagerAccountId();

    my $Url = EPAGESJ_BASE_URI . "accounts/ads/$AdsManagerAccountId/sub-accounts/$AdsAccountId/merchant-center-link/enable";
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {};

    LogGoogleSmartShopping('enableGoogleAdsLink request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('enableGoogleAdsLink response: ', {
        Response => $hResponse,
    });

    return;
}

#========================================================================================
# function     createGoogleAdsCampaign
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleAdsCampaign($Email, $Name);
#----------------------------------------------------------------------------------------
# description  Creates GoogleAds Campaign
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns"
#----------------------------------------------------------------------------------------
# input       $AdsSubAccountId | string with Ads Account Id | string
# input       $MerchantCenterAccountId | string with MerchantCenterAccountId | string
# input       $DailyBudget | string with DailyBudget | string
# input       $Name | string with campaign name | string
#========================================================================================
sub createGoogleAdsCampaign {
    my $self = shift;
    my ($hParams) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $hParams->{'AdsSubAccountId'}
    ) ;
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        name =>  $hParams->{'AdsCampaignName'}." Campaign",
        salesCountry => $self->_getSalesCountry() ,
        dailyBudget => {
            name => $hParams->{'AdsCampaignName'}." Budget",
            amount =>  $hParams->{'DailyBudget'},
            currency => "EUR"
        },
        merchantCenterAccountId => $hParams->{'MerchantCenterAccountId'},
    };

    LogGoogleSmartShopping('createGoogleAdsCampaign  request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleAdsCampaign response: ', {
        Response => $hResponse,
    });

    if (defined($hResponse) && defined($hResponse->{'id'})) {
        $self->{'GoogleSmartShoppingConfig'}->set({'AdsCampaignId' => $hResponse->{'id'}});
    }

    return $hResponse;
}

#========================================================================================
# function     getSmartShoppingCampaigns
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aResponse = $self->getGoogleAdsAccounts();
#----------------------------------------------------------------------------------------
# description  Returns SmartShoppingCampaigns array,
#               or empty array if no campaigns found.
#----------------------------------------------------------------------------------------
# return       $aResponse | array with Google Ads Campaigns | array.ref
#========================================================================================
sub getGetSmartShoppingCampaigns {
    my $self = shift;
    my ($hParams) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $hParams->{'AdsSubAccountId'}
    ) ;
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    LogGoogleSmartShopping('getGetSmartShoppingCampaigns  request: ', {
        Url => $Url
    });

    my $aResponse = $self->doGet($Url, $hHeaders);

    LogGoogleSmartShopping('getGetSmartShoppingCampaigns response: ', {
        Response => $aResponse,
    });

    return $aResponse;
}

#========================================================================================
# function     updateGoogleAdsCampaign
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateGoogleAdsCampaign($CampaignStatusEnabled);
#----------------------------------------------------------------------------------------
# description  Updates status of GoogleAds Campaign
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns"
#----------------------------------------------------------------------------------------
# input       $hCampaignStatusUpdate | hash with campaign Id and status enabled/disabled value | hash.ref
#========================================================================================
sub updateGoogleAdsCampaign {
    my $self = shift;
    my ($hCampaignStatusUpdate) = @_;

    my $ManagerAdsAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s",
        EPAGESJ_BASE_URI,
        $ManagerAdsAccountId,
        $AdsAccountId,
        $hCampaignStatusUpdate->{'CampaignId'}
    );
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };
    my $hPayload = { 'status' => CAMPAIGN_STATUS_STRINGS->{$hCampaignStatusUpdate->{'CampaignStatusEnabled'}}};

    LogGoogleSmartShopping('updateGoogleAdsCampaign  request: ', {
        Url => $Url,
        Payload => $hPayload,
    });
    $self->doPut($Url, $hHeaders, $hPayload);

    return;
}

#========================================================================================
# function     updateGoogleAdsCampaignBudget
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->updateGoogleAdsCampaignBudget($hCampaignBudgetUpdate);
#----------------------------------------------------------------------------------------
# description  Updates budget of GoogleAds Campaign
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns/{campapaignId}/daily-budget"
#----------------------------------------------------------------------------------------
# input       $hCampaignBudgetUpdate | hash with campaign Id and budget value | hash.ref
#========================================================================================
sub updateGoogleAdsCampaignBudget {
    my $self = shift;
    my ($hCampaignBudgetUpdate) = @_;

    my $ManagerAdsAccountId = $self->_getAdsManagerAccountId();
    my $AdsAccountId = $self->_getAdsAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/%s/daily-budget",
        EPAGESJ_BASE_URI,
        $ManagerAdsAccountId,
        $AdsAccountId,
        $hCampaignBudgetUpdate->{'AdsCampaignId'}
    );
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };
    my $hPayload = {
        'amount' => $hCampaignBudgetUpdate->{'DailyBudget'},
        'currency' => $hCampaignBudgetUpdate->{'DailyBudgetCurrency'},
    };

    LogGoogleSmartShopping('updateGoogleAdsCampaignBudget  request: ', {
        Url => $Url,
        Payload => $hPayload,
    });
    $self->doPut($Url, $hHeaders, $hPayload);

    return;
}

#========================================================================================
# function     createGoogleConversionAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = $self->createGoogleConversionAction($Email, $Name);
#----------------------------------------------------------------------------------------
# description  Creates Google Ads Conversion Action
#               ads/{managerId}/sub-accounts/{subAccountId}/smart-shopping-campaigns/conversion-actions"
#----------------------------------------------------------------------------------------
# input       $AdsSubAccountId | string with Ads Account Id | string
# input       $MerchantCenterAccountId | string with MerchantCenterAccountId | string
# input       $DailyBudget | string with DailyBudget | string
# input       $Name | string with campaign name | string
#========================================================================================
sub createGoogleConversionAction {
    my $self = shift;
    my ($hParams) = @_;

    my $ManagerAccountId = $self->_getAdsManagerAccountId();
    my $Url = sprintf("%saccounts/ads/%s/sub-accounts/%s/smart-shopping-campaigns/conversion-actions",
        EPAGESJ_BASE_URI,
        $ManagerAccountId,
        $hParams->{'AdsSubAccountId'}
    ) ;
    my $hHeaders = {
        'AdminToken' => $self->_createAdminToken(),
        'X-DeveloperToken' => $self->_getDeveloperToken()
    };

    my $hPayload = {
        name =>  $hParams->{'AdsCampaignName'}. ' - ' . $hParams->{'AdsCampaignId'} . ' Conversion Action',
    };

    LogGoogleSmartShopping('createGoogleConversionAction request: ', {
        Url => $Url,
        Payload => $hPayload,
    });

    my $hResponse = $self->doPost($Url, $hHeaders, $hPayload);

    LogGoogleSmartShopping('createGoogleConversionAction response: ', {
        Response => $hResponse,
    });

    return $hResponse;
}

#----------------------------------------------------------------------------------------
sub _getSalesCountry {
    my $self = shift;
    my $Shop = $self->{'Shop'};
    return $Shop->get('Address')->get('Country')->{'Code2'};
}

#----------------------------------------------------------------------------------------
sub _createAdminToken {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->GetAccessToken($self->{'GoogleSmartShopping_AdminRefreshToken'});
}

#----------------------------------------------------------------------------------------
sub _getCurrencyCode {
    my $self = shift;
    my $Shop =$self->{'Shop'};
    return $Shop->get('CurrencyID');
}

#----------------------------------------------------------------------------------------
sub _getTimeZone {
    my $self = shift;
    return GetLocalTimeZone();
}

#----------------------------------------------------------------------------------------
sub _getDeveloperToken {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->{'GoogleSmartShopping_AdsDeveloperToken'};
}

#----------------------------------------------------------------------------------------
sub _getAdsManagerAccountId {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->{'GoogleSmartShopping_AdsManagerAccountId'};
}

#----------------------------------------------------------------------------------------
sub _getAdsAccountId {
    my $self = shift;

    $self->loadGoogleSettings();
    my $GoogleSmartShoppingConfig = $self->{'GoogleSmartShoppingConfig'};
    return $GoogleSmartShoppingConfig->get('GoogleAdsAccountID');
}

#----------------------------------------------------------------------------------------
sub _getEpagesMultiCLientAccountId {
    my $self = shift;

    $self->loadGoogleSettings();
    return $self->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'};
}

1;