#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Utilities used in different modules of GoogleSmartShopping cartridge.
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::Tools;
use base qw (Exporter);
use strict;

use JSON::MaybeXS qw( decode_json );

use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::ThirdPartyConfig::API::Log qw (Log3rdParty);
use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    GOOGLE_ERROR_CODES
    ERROR_UNKNOWN
    ERROR_TOKEN_EXPIRED_OR_REVOKED
    ERROR_INVALID_TOKEN
    ERROR_UNAUTHENTICATED
);

our @EXPORT_OK = qw (
    LogGoogleSmartShopping
    GetErrorCode
    GetGoogleApiErrorCode
);

#========================================================================================
# function     LogGoogleSmartShopping
#----------------------------------------------------------------------------------------
# syntax       LogGoogleSmartShopping($Name, $Debug, $Section);
#----------------------------------------------------------------------------------------
# example      LogGoogleSmartShopping('SendRequestHash', \%Hash, 'Send');
#----------------------------------------------------------------------------------------
# description  Logs a structure to the communication.log.
#----------------------------------------------------------------------------------------
# input        $Name | name of data | string
# input        $Debug | data to be logged | hash
# input        $Section | name of the section to be logged (optional - defaults to name of calling sub) | string
# input        $aFilter | (optional) filter (perl regular expression for
#                       s/$Filter[^\n]+\n/\n/gis) it can also be a CodeReference see Log3rdParty| ref.array.string
#========================================================================================
sub LogGoogleSmartShopping($$;$$) {
    my ($Name, $DebugStuff, $Section, $Filter) = @_;

    TestString('Section', $Section) if defined $Section;
    $Section //= (split(/::/, (caller(1))[3]))[-1];

    return unless LoadRootObject()->get('GoogleSmartShopping_Logging');
    Log3rdParty($Name, $DebugStuff, 'GoogleSmartShopping::'.$Section, $Filter);

    return;
}

#========================================================================================
# function     GetErrorCode
#----------------------------------------------------------------------------------------
# syntax       GetErrorCode($Error);
#----------------------------------------------------------------------------------------
# example      GetErrorCode($Error);
#----------------------------------------------------------------------------------------
# description  returns an internal error code defined in
#               DE_EPAGES::GoogleSmartShopping::API::Constants
#----------------------------------------------------------------------------------------
# input        $ErrorObject | complete Error object returned by GetError() | string
#----------------------------------------------------------------------------------------
# return       $ErrorType | internal error code | string
#========================================================================================
sub GetErrorCode($) {
    my ($ErrorObject) = @_;

    my $JSONResponse = $ErrorObject->vars->{'Response'};
    unless (defined($JSONResponse)) {
        return ERROR_UNKNOWN;
    }

    my $hResponse;
    eval {
        $hResponse = decode_json($JSONResponse);
    };
    if (ExistsError()) {
        LogGoogleSmartShopping("GetErrorCode(): Error contained invalid JSON, ", $JSONResponse);
        return ERROR_UNKNOWN;
    }

    unless (ref($hResponse) eq 'HASH') {
        LogGoogleSmartShopping("GetErrorCode(): unrecognized error format, ", $hResponse);
        return ERROR_UNKNOWN;
    }

    my $ErrorId = $hResponse->{'errorId'};
    if (defined($ErrorId)) {
        if ($ErrorId eq 'google-api-error') {
            return GetGoogleApiErrorCode($hResponse->{'details'});
        } else {
            return ERROR_UNKNOWN;
        }
    }
    my $Error = $hResponse->{'error'};
    my $Description = $hResponse->{'error_description'};
    if (defined($Error)) {
        if ($Error eq 'invalid_grant' && defined($Description)
        && $Description eq "Token has been expired or revoked.") {
            return ERROR_TOKEN_EXPIRED_OR_REVOKED;
        } elsif ($Error eq 'invalid_request' && defined($Description)
            && $Description eq "Invalid Credentials") {
            return ERROR_UNAUTHENTICATED;
        } elsif ($Error eq 'invalid_token') {
            return ERROR_INVALID_TOKEN;
        }
    }

    return ERROR_UNKNOWN;
}

#========================================================================================
# function     GetGoogleApiErrorCode
#----------------------------------------------------------------------------------------
# syntax       GetGoogleApiErrorCode($hErrorDetails);
#----------------------------------------------------------------------------------------
# example      GetGoogleApiErrorCode($hErrorDetails);
#----------------------------------------------------------------------------------------
# description  returns an internal error code
#----------------------------------------------------------------------------------------
# input        $hErrorDetails | detailed response object from Google | string
#----------------------------------------------------------------------------------------
# return       $ErrorType | internal error code | string
#========================================================================================
sub GetGoogleApiErrorCode($) {
    my ($hErrorDetails) = @_;

    my $ErrorList = $hErrorDetails->{'errors'};
    if (defined($ErrorList)) {
        foreach my $hError (@{ $ErrorList }) {
            my $Reason = $hError->{'reason'};
            next unless $Reason;
            my $Code = GOOGLE_ERROR_CODES->{$Reason};
            next unless $Code;
            return $Code;
        }
    }
    return ERROR_UNKNOWN;
}



1;
