#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::GoogleSmartShoppingConfig
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::GoogleSmartShoppingConfig;
use base DE_EPAGES::ProductPortal::UI::PortalConfig;

use strict;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( MERCHANT_CENTER_REQUIREMENTS_CHECK_LIST );
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use HTTP::Status qw ( HTTP_NOT_FOUND HTTP_OK );
use DE_EPAGES::WebService::API::Util qw (ReadJSONBoolean);

#==============================================================================
sub Save {
    my $self    = shift;
    my $Servlet = shift;

    my $Form = $Servlet->form;
    my $DescriptionAttributeID = $Form->formValue('DescriptionAttribute');
    unless (isValid($DescriptionAttributeID) && $DescriptionAttributeID =~ /^\d+$/) {
        $Form->addFormError({
            'Form'  => 'GoogleSmartShoppingConfigSettings',
            'Reason'=> 'NoDescriptionAttribute',
            'Name'  => 'DescriptionAttribute'
        });
    }
    foreach my $field (qw(Login Password)) {
        if (!isValid($Form->formValue($field))) {
            $Form->addFormError({
                'Form'  => 'GoogleSmartShoppingConfigSettings',
                'Reason'=> 'MandatoryFieldMissing',
                'Name'  => $field
            });
        }
    }

    return $self->SUPER::Save($Servlet);
}

#========================================================================================
# function     HandleChecklistAJAX
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateChecklist($Servlet)
#----------------------------------------------------------------------------------------
# description  This ChangeAction method is called by AJAX in the GoogleSmartShopping
#               setup tab page to update saved values in the backend for the
#               GoogleSmartShopping Merchant Center Account Requirements checklist.
#               It will also return the current status of the checklist, and can be
#               called either with POST (to update) or GET (to only retrieve the status).
#----------------------------------------------------------------------------------------
# input        POST data contains MERCHANT_CENTER_REQUIREMENTS_CHECK_LIST as possible
#               keys, this is defined in DE_EPAGES/GoogleSmartShopping/API/Constants.pm,
#               values are boolean.
#----------------------------------------------------------------------------------------
# output       Hash reference containing keys MERCHANT_CENTER_REQUIREMENTS_CHECK_LIST
#               defined in DE_EPAGES/GoogleSmartShopping/API/Constants.pm, values are
#               boolean. Cartridges magically converts this to JSON for us, because we
#               set 'ContentType' to 'application/json' before calling
#               $Servlet->writeContentOutput().
#----------------------------------------------------------------------------------------
# return       null - success/failure indicated by $Servlet->response->code()
#========================================================================================
sub HandleChecklistAJAX {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    unless ($self->_existsGoogleSmartShoppingConfig($Shop)) {
        $Servlet->response->code(HTTP_NOT_FOUND);
        return;
    }

    my $Form = $Servlet->form();
    my $hPostData = $Form->formValues();


    LogGoogleSmartShopping("HandleChecklistAJAX POST data:", $hPostData);
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    foreach my $key (MERCHANT_CENTER_REQUIREMENTS_CHECK_LIST) {
        if (defined $hPostData->{$key}) {
            # The AJAX serialization to JSON converts JS boolean true/false
            $GoogleSmartShoppingConfig->set({
                $key => ReadJSONBoolean($hPostData->{$key}->[0])
            });
        }
    }

    my $hRequirementsStatus = $GoogleSmartShoppingConfig->getGoogleMerchantCenterRequirementsStatus();
    LogGoogleSmartShopping("HandleChecklistAJAX response:", $hRequirementsStatus);
    $Servlet->params('ContentType', 'application/json');
    $Servlet->response->code(HTTP_OK);
    $Servlet->writeContentOutput($hRequirementsStatus);
}

#========================================================================================
# function     ConnectMerchantCenterAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       ConnectMerchantCenterAccount($Servlet)
#----------------------------------------------------------------------------------------
# description  This ChangeAction method is called by clicking "Connect" button from "Google
#               Merchant Center account" section in the GoogleSmartShopping
#               setup tab page. It performs shop's URL site verification and claiming against
#               selected Merchant Center account. On success, Merchant Center account is saved
#               in the DataBase.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | Servlet
#========================================================================================
sub ConnectMerchantCenterAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    return unless $self->_existsGoogleSmartShoppingConfig($Shop);

    my $Form = $Servlet->form();
    my $hPostData = $Form->formValues();
    my $SelectedMerchantCenterAccountOption = $hPostData->{'MerchantCenterAccountList'}->[0];
    return unless (isValid($SelectedMerchantCenterAccountOption));

    my @MerchantCenterAccountParts = split(/\|/, $SelectedMerchantCenterAccountOption, 2);
    my $MerchantCenterAccountName = pop @MerchantCenterAccountParts;
    my $MerchantCenterAccountId = join('', @MerchantCenterAccountParts);
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    if (
        $self->_saveMerchantCenterAccountInfo($Shop, $GoogleSmartShoppingConfig, $MerchantCenterAccountId, $MerchantCenterAccountName)
        && $self->_updateMerchantCenterShopUrl($Shop))
    {
        $self->_verifyAndClaimWebsite($Servlet, $Shop);
    } else {
        LogGoogleSmartShopping('Error on saveMerchantCenterAccountInfo', {});
    }

    return;
}

#========================================================================================
# function     VerifyAndClaimWebsite
# state        public
#----------------------------------------------------------------------------------------
# syntax       VerifyAndClaimWebsite( $Servlet );
#----------------------------------------------------------------------------------------
# description  Verify And Claim Website called by create new merchant center account
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
#========================================================================================
sub VerifyAndClaimWebsite {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    $self->_verifyAndClaimWebsite($Servlet, $Shop);

    return;
}

#========================================================================================
sub _verifyAndClaimWebsite {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

    eval {
        $self->_doSiteVerification($Servlet, $Shop, $GoogleApiService);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Error verifying website: ', $Error->message());
        return;
    }

    my $hClaimResponse;
    eval {
        $hClaimResponse = $GoogleApiService->claimWebsite();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Error claiming website: ', $Error->message());
    }
    return $hClaimResponse;
}

#========================================================================================
sub _doSiteVerification {
    my $self = shift;
    my ($Servlet, $Shop, $GoogleApiService) = @_;

    my $VerificationToken = $self->_getSiteVerificationToken($Shop, $GoogleApiService);

    $self->_saveSiteVerificationToken($Shop, $VerificationToken);
    $self->_triggerSiteVerification($Servlet, $Shop, $GoogleApiService);

    return;
}
#========================================================================================
sub _saveSiteVerificationToken {
    my $self = shift;
    my ($Shop, $VerificationToken) = @_;

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    $GoogleSmartShoppingConfig->set({ 'SiteVerificationToken' => $VerificationToken });
}
#========================================================================================
sub _getSiteVerificationToken {
    my $self = shift;
    my ($Shop, $GoogleApiService) = @_;

    my $ShopURL = $Shop->get('WebUrlUnitySSL');
    unless (defined $ShopURL) {
        Error('Missing WebUrlUnitySSL param in Shop');
    }

    my $hResponse = $GoogleApiService->getSiteVerificationToken($ShopURL);
    return $hResponse->{'token'};
}

#========================================================================================
sub _triggerSiteVerification {
    my $self = shift;
    my ($Servlet, $Shop, $GoogleApiService) = @_;

    my $ShopURL = $Shop->get('WebUrlUnitySSL');
    unless (defined $ShopURL) {
        Error('Missing WebUrlUnitySSL param in Shop');
    }

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    my $VerificationToken = $GoogleSmartShoppingConfig->get('SiteVerificationToken');
    unless (defined $VerificationToken) {
        Error('Missing GoogleSiteVerificationToken param in Shop');
    }

    return $GoogleApiService->triggerSiteVerification($ShopURL, $VerificationToken);
}

#========================================================================================
sub _existsGoogleSmartShoppingConfig {
    my $self = shift;
    my ($Shop) = @_;

    my $CallingFunction = ( caller(0) )[3];
    unless ($Shop->existsChild('GoogleSmartShoppingConfig')) {
        LogGoogleSmartShopping("GoogleSmartShoppingConfig shop object not found in ", $CallingFunction);
        return 0;
    }
    return 1;
}

#========================================================================================
sub _saveMerchantCenterAccountInfo {
    my $self = shift;
    my ($Shop, $GoogleSmartShoppingConfig, $MerchantCenterAccountId, $MerchantCenterAccountName) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({ Shop => $Shop });
    my $hUserAccountInfo = $Service->getUserAccountInfo();

    return 0 unless (isValid($hUserAccountInfo) && isValid($hUserAccountInfo->{'accountIdentifiers'}));

    my $aAccountIdentifiers = $hUserAccountInfo->{'accountIdentifiers'};
    my @MerchanCenterAccountInfo = grep { $_->{'merchantId'} eq $MerchantCenterAccountId } @$aAccountIdentifiers;
    my $hMerchanCenterAccountInfo = pop @MerchanCenterAccountInfo;

    unless (isValid($hMerchanCenterAccountInfo)) {
        LogGoogleSmartShopping('Merchant Center account info has no selected ID: ', $MerchantCenterAccountId);
        return 0;
    }
    my $EpagesAggregatorId = LoadRootObject()->get('GoogleSmartShopping_MerchantCenterMultiClientAccountID');
    my $IsMerchantCenterSubAccount = (
        isValid($hMerchanCenterAccountInfo->{'aggregatorId'}) &&
        ($hMerchanCenterAccountInfo->{'aggregatorId'} eq $EpagesAggregatorId)
    ) ? 1 : 0;
    $GoogleSmartShoppingConfig->set({
        'MerchantCenterAccountID' => $hMerchanCenterAccountInfo->{'merchantId'},
        'MerchantCenterAccountName' => $MerchantCenterAccountName,
        'MerchantCenterMultiClientAccountID' => $hMerchanCenterAccountInfo->{'aggregatorId'},
        'IsMerchantCenterSubAccount' => $IsMerchantCenterSubAccount
    });
    LogGoogleSmartShopping('Saved Merchant Center account info: ', $hMerchanCenterAccountInfo);

    return 1;
}

sub _updateMerchantCenterShopUrl {
    my $self = shift;
    my ($Shop) = @_;

    my $ShopUrl = $Shop->get('WebUrlUnitySSL');

    LogGoogleSmartShopping('Epages Shop Url: ', $ShopUrl);
    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({ Shop => $Shop });
    eval {
        $Service->updateMerchantCenterShopUrl($ShopUrl);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Merchant Center Shop URL Update failed', {
            'Error' => $Error->message(),
        });
        return 0;
    }

    return 1;
}
1;
