#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_Campaigns
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping Ads Campaign creation
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_Campaigns;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( String HashRef );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::More;

use constant SHOP_NAME => 'Spotlight';

use constant MERCHANT_CENTER_ID => 503220055;
use constant CONNECTED_GOOGLE_ADS_ID => 1133146331;
use constant ADS_CAMPAIGN_DAILY_BUDGET_INPUT => "12,35";
use constant ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT => 12.35;
use constant ADS_CAMPAIGN_DAILY_BUDGET => 12.35;
use constant ADS_CAMPAIGN_DAILY_BUDGET_CURRENCY => 'EUR';
use constant ADS_CAMPAIGN_ID => 14886962060;

use constant MOCKED_CAMPAIGN_SERVICE_RESPONSE_PAST_DATE => [
    {
        'id' => ADS_CAMPAIGN_ID,
        'dailyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2021-10-07',
        'status' => 'PAUSED'
    }
];
use constant MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE => [
    {
        'id' => ADS_CAMPAIGN_ID,
        'dailyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2121-10-07',
        'status' => 'PAUSED'
    }
];
use constant EXPECTED_GET_CAMPAIGNS_RESPONSE_FUTURE_DATE => [
    {
        'id' => ADS_CAMPAIGN_ID,
        'dailyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2121-10-07T00:00:00',
        'status' => 'PAUSED',
        'isFuture' => '1'
    }
];

use constant ADS_CAMPAIGN_RESPONSE_ERROR => 'UNAUTHORIZED';

sub test {
    my $self = shift;

    $self->test_CreateGoogleAdsCampaign_Success();
    $self->test_GetGoogleAdsCampaigns_SuccessPast();
    $self->test_GetGoogleAdsCampaigns_SuccessFuture();
    $self->test_SwitchGoogleAdsCampaignStatus_Success();
    $self->test_SwitchGoogleAdsCampaignStatus_Error();
    $self->test_UpdateGoogleAdsCampaignBudget_Success();

    done_testing();
}


sub test_CreateGoogleAdsCampaign_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $hFormQueryParameters = {
        'DailyBudget' => [ADS_CAMPAIGN_DAILY_BUDGET_INPUT]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'createGoogleAdsCampaign' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE->[0]
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $SUT = $self->_createSUT($ApiAdsService, $ScriptTagService);

    my $Response = $SUT->CreateGoogleAdsCampaign($Servlet);

    ok( WasCalled($SUT, '_validateCreateGoogleAdsCampaignRequest'), $SubTestName . ' - Verify that _validateCreateGoogleAdsCampaignRequest was called.');
    ok( WasCalled($SUT, '_createCampaign'), $SubTestName . ' - Verify that _createCampaign was called.');
    ok( WasCalled($SUT, '_createConversionAction'), $SubTestName . ' - Verify that _createConversionAction was called.');
    ok( WasCalled($ScriptTagService, 'addScriptTag'), $SubTestName . ' - Verify that addScriptTag was called.');
    ok( WasCalled($ApiAdsService, 'createGoogleAdsCampaign'), $SubTestName . ' - Verify that service method createGoogleAdsCampaign was called.');
    ok( WasCalled($ApiAdsService, 'createGoogleConversionAction'), $SubTestName . ' - Verify that service method createGoogleConversionAction was called.');
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleAdsCampaign');

    my $hExpectedCampaignCreationParams = {
        'AdsCampaignName' => SHOP_NAME,
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID,
        'DailyBudget' => ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT,
        'MerchantCenterAccountId' => MERCHANT_CENTER_ID
    };
    is_deeply( @$aParams[0], $hExpectedCampaignCreationParams, "$SubTestName - Verify that correct params passed on createGoogleAdsCampaign service call." );

    $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleConversionAction');
    my $hExpectedCampaignConversionActionParams = {
        'AdsCampaignName' => SHOP_NAME,
        'AdsCampaignId' => ADS_CAMPAIGN_ID,
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID,
    };
    is_deeply( @$aParams[0], $hExpectedCampaignConversionActionParams, "$SubTestName - Verify that correct params passed on createGoogleConversionAction service call." );

    is( $Response, undef, "$SubTestName - Verify that AdsCampaign created and response contains no error");
}

sub test_GetGoogleAdsCampaigns_SuccessPast {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'getGetSmartShoppingCampaigns' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_PAST_DATE
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $SUT = $self->_createSUT($ApiAdsService, $ScriptTagService);

    my $aResponse = $SUT->GetGoogleAdsCampaigns($Shop, $GoogleSmartShoppingConfig);

    ok( WasCalled($ApiAdsService, 'getGetSmartShoppingCampaigns'), $SubTestName . ' - Verify that service method getGetSmartShoppingCampaigns was called.');
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'getGetSmartShoppingCampaigns');

    my $hExpectedParams = {
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID
    };

    is_deeply( @$aParams[0], $hExpectedParams, "$SubTestName - Verify that correct params passed on getGetSmartShoppingCampaigns service call." );

    is_deeply( $aResponse, MOCKED_CAMPAIGN_SERVICE_RESPONSE_PAST_DATE, "$SubTestName - Verify that GetGoogleAdsCampaigns returns expected response");

}

sub test_GetGoogleAdsCampaigns_SuccessFuture {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'getGetSmartShoppingCampaigns' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $SUT = $self->_createSUT($ApiAdsService, $ScriptTagService);

    my $aResponse = $SUT->GetGoogleAdsCampaigns($Shop, $GoogleSmartShoppingConfig);

    ok( WasCalled($ApiAdsService, 'getGetSmartShoppingCampaigns'), $SubTestName . ' - Verify that service method getGetSmartShoppingCampaigns was called.');
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'getGetSmartShoppingCampaigns');

    my $hExpectedParams = {
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID
    };

    is_deeply( @$aParams[0], $hExpectedParams, "$SubTestName - Verify that correct params passed on getGetSmartShoppingCampaigns service call." );

    is_deeply( $aResponse, EXPECTED_GET_CAMPAIGNS_RESPONSE_FUTURE_DATE, "$SubTestName - Verify that GetGoogleAdsCampaigns returns expected response");

}

sub test_SwitchGoogleAdsCampaignStatus_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'CampaignId'            => [ ADS_CAMPAIGN_ID ],
        'CampaignStatusEnabled' => [ 'false' ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaign' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $SUT = $self->_createSUT($ApiAdsService, $ScriptTagService);

    $SUT->SwitchCampaignStatus($Servlet);

    my $hExpectedSwitchCampaignParams = {
        'CampaignId' => ADS_CAMPAIGN_ID,
        'CampaignStatusEnabled' => 0
    };
    ok(
        WasCalled($ApiAdsService, 'updateGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method updateGoogleAdsCampaign was called.'
    );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaign');
    is_deeply(
        @$aParams[0],
        $hExpectedSwitchCampaignParams,
        "$SubTestName - Verify that correct params passed on updateGoogleAdsCampaign service call."
    );

    ok(
        WasCalled($Servlet, 'writeContentOutput'),
        "$SubTestName - Verify that servlet output writing method was called."
    );
    $aParams = GetParametersFromMockifyCall($Servlet, 'writeContentOutput');
    is_deeply(
        @$aParams[0],
        $hExpectedSwitchCampaignParams,
        "$SubTestName - Verify that correct params passed on writeContentOutput servlet call."
    );
}

sub test_SwitchGoogleAdsCampaignStatus_Error {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'CampaignId'            => [ ADS_CAMPAIGN_ID ],
        'CampaignStatusEnabled' => [ 'true' ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaign' => 'Error'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $SUT = $self->_createSUT($ApiAdsService, $ScriptTagService);

    $SUT->SwitchCampaignStatus($Servlet);

    ok( WasCalled($ApiAdsService, 'updateGoogleAdsCampaign'), $SubTestName . ' - Verify that service method updateGoogleAdsCampaign was called.');
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaign');
    is_deeply(
        @$aParams[0],
        { 'CampaignId' => ADS_CAMPAIGN_ID, 'CampaignStatusEnabled' => 1 },
        "$SubTestName - Verify that correct params and values are passed on updateGoogleAdsCampaign service call."
    );

    ok(
        WasCalled($Servlet, 'writeContentOutput'),
        "$SubTestName - Verify that servlet output writing method was called."
    );
    $aParams = GetParametersFromMockifyCall($Servlet, 'writeContentOutput');
    ok(
        defined $aParams->[0]->{'Error'},
        "$SubTestName - Verify that error param is present on writeContentOutput servlet call."
    );
}


sub test_UpdateGoogleAdsCampaignBudget_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'AdsCampaignId' => [ ADS_CAMPAIGN_ID ],
        'DailyBudget' => [ ADS_CAMPAIGN_DAILY_BUDGET ],
        'DailyBudgetCurrency' => [ ADS_CAMPAIGN_DAILY_BUDGET_CURRENCY ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaignBudget' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $SUT = $self->_createSUT($ApiAdsService, $ScriptTagService);

    $SUT->UpdateGoogleAdsCampaign($Servlet);

    my $hExpectedUpdateCampaignParams = {
        'AdsCampaignId'       => ADS_CAMPAIGN_ID,
        'DailyBudget'         => ADS_CAMPAIGN_DAILY_BUDGET,
        'DailyBudgetCurrency' => ADS_CAMPAIGN_DAILY_BUDGET_CURRENCY
    };
    ok(
        WasCalled($SUT, '_validateUpdateGoogleAdsCampaignRequest'),
        $SubTestName . ' - Verify that _validateUpdateGoogleAdsCampaignRequest was called.'
    );

    ok(
        WasCalled($ApiAdsService, 'updateGoogleAdsCampaignBudget'),
        $SubTestName . ' - Verify that service method updateGoogleAdsCampaignBudget was called.'
    );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaignBudget');
    is_deeply(
        @$aParams[0],
        $hExpectedUpdateCampaignParams,
        "$SubTestName - Verify that correct params passed on updateGoogleAdsCampaignBudget service call."
    );

}


#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($AdsApiService, $ScriptTagService) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::Campaigns');

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySut->spy('_validateCreateGoogleAdsCampaignRequest')->whenAny();
    $MockifySut->spy('_validateUpdateGoogleAdsCampaignRequest')->whenAny();
    $MockifySut->spy('_createCampaign')->whenAny();
    $MockifySut->spy('_createConversionAction')->whenAny();

    if (defined($AdsApiService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($AdsApiService);
    }

    $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::ScriptTagService::new')
        ->when(String(), HashRef())
        ->thenReturn($ScriptTagService);

    return $MockifySut->getMockObject();
}

sub _createScriptTagService {
    my $self = shift;
    my ($Shop) = @_;

    my $aParameterList = [{'Shop' => $Shop}];

    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::GoogleSmartShopping::API::ScriptTagService',
                   $aParameterList
              );
    $Mockify->mock('addScriptTag')->when()->thenReturnUndef();
    $Mockify->mock('deleteScriptTag')->when()->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        ChecklistBillingTOC        => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo       => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice      => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy      => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountID    => $hConfig->{'MerchantCenterAccountID'} // undef,
        GoogleAdsAccountID         => $hConfig->{'GoogleAdsAccountID'} // undef
    };
    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $MockifyConfig->getMockObject();
}

sub _createGoogleAdsApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService');
    $Mockify->mock('createGoogleAdsCampaign')
        ->when(HashRef())
        ->thenReturn($hServiceResponse->{'createGoogleAdsCampaign'} // {});

    $Mockify->mock('getGetSmartShoppingCampaigns')
        ->when(HashRef())
        ->thenReturn($hServiceResponse->{'getGetSmartShoppingCampaigns'} // {});

    $Mockify->mock('createGoogleConversionAction')
        ->when(HashRef())
        ->thenReturn($hServiceResponse->{'createGoogleConversionAction'} // {});

    if (defined $hServiceResponse->{'updateGoogleAdsCampaign'}) {
        if ($hServiceResponse->{'updateGoogleAdsCampaign'} eq 'Success') {
            $Mockify->mock('updateGoogleAdsCampaign')
                ->when(HashRef())
                ->thenReturnUndef();
        } else {
            $Mockify->mock('updateGoogleAdsCampaign')
                ->when(HashRef())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    if (defined $hServiceResponse->{'updateGoogleAdsCampaignBudget'}) {
        if ($hServiceResponse->{'updateGoogleAdsCampaignBudget'} eq 'Success') {
            $Mockify->mock('updateGoogleAdsCampaignBudget')
                ->when(HashRef())
                ->thenReturnUndef();
        } else {
            $Mockify->mock('updateGoogleAdsCampaignBudget')
                ->when(HashRef())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
    $MockifyServlet->addMockWithReturnValue('response', $self->_createServletResponse());
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, ['hashref']);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;
    my $hAttributes = {
        'ID'                          => undef,
        'GoogleSiteVerificationToken' => 'VerificationToken',
        'GoogleSmartShoppingConfig'   => $GoogleSmartShoppingConfig,
        'LanguageID'                  => 'DE',
        'NameOrAlias'                 => SHOP_NAME,
        'CurrencyID'                  => 'EUR',
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'insertChild', $GoogleSmartShoppingConfig, [ {'string' => 'GoogleSmartShoppingConfig'}, 'hashref' ]
    );

    return $MockifyShop->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('values')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('field')
        ->whenAny()
        ->thenReturn(ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT);

    $Mockify->mock('executeFormError')
        ->when()
        ->thenReturnUndef();

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    $Mockify->mock('addFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createServletResponse {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'HTTP::Response');
    $Mockify->mock('code')
        ->whenAny()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

