#========================================================================================
# package      DE_EPAGES::Object::API::PageCache
# state        public
# description  Interface for page cache handling.
#========================================================================================
package DE_EPAGES::Object::API::PageCache;

use strict;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::TestParameter qw( TestObject TestPackage TestString );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::PageCache::File;
use DE_EPAGES::Object::API::PageCache::Varnish;
use DE_EPAGES::Object::API::PageCache::ZipFile;
use DE_EPAGES::Object::API::PageCache::LZ4File;
use DE_EPAGES::Object::API::PageCache::ZStdFile;
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use constant SECONDS_PER_MINUTE => 60;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageCache = DE_EPAGES::Object::API::PageCache->new( %Options );
#----------------------------------------------------------------------------------------
# example      $PageCache = DE_EPAGES::Object::API::PageCache->new( Site => $Site );
#----------------------------------------------------------------------------------------
# description  Creates a PageCache object for a Site.
#----------------------------------------------------------------------------------------
# input        %Options | options <ul>
#                       <li>Site - Site object - object </li>
#                   </ul>
# return       $PageCache | PageCache object | DE_EPAGES::Object::API::PageCache
#========================================================================================
sub new {
    my $class = shift;
    my (%Options) = @_;
    TestObject('Site', $Options{'Site'}, 'DE_EPAGES::Object::API::Object::Site');

    if (! defined($Options{'Backend'})) {
        my $EpagesConfig = EpagesConfig();
        my $Backend = $EpagesConfig->get('DE_EPAGES::Object', 'PageCacheBackend') // 'File';
        $Backend = "DE_EPAGES::Object::API::PageCache::$Backend";
        TestPackage($Backend);
        $Options{'EpagesConfig'} = $EpagesConfig;
        $Options{'Backend'} = $Backend->new( %Options );
    }

    return bless { %Options }, $class;
}

sub _backend { ## no critic qw(RequireArgUnpacking)
    return shift->{Backend};
}

sub _site { ## no critic qw(RequireArgUnpacking)
    return shift->{Site};
}

#========================================================================================
# function     compute
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($Content, LastModified) = $PageCache->compute($Object, $CacheKeyPart, $IfModifiedSince, $Sub);
#----------------------------------------------------------------------------------------
# example      ($Content, LastModified) = $PageCache->compute($Object, 'View_en_GB',
#                   time() - 100, sub { return "Today is: ".scalar(localtime), 1 };
#               );
#----------------------------------------------------------------------------------------
# description  Gets a page from the cache or computes a new page and stores it in the
#               cache.
#               Returns $Content = undef if the content has not been modified since
#               $IfModifiedSince.
#----------------------------------------------------------------------------------------
# input        $Object | object | DE_EPAGES::Object::API::Object::Object
# input        $CacheKeyPart | cache key unique for $Object | string
# input        $IfModifiedSince | (optional) epoch value of last modification
#                       from the HTTP header If-Modified-Since | int
# input        $Sub | function to compute the page content if the page is not already
#                       cached. This function returns 2 values: $Content and $IsCacheable.
#                       If $IsCacheable is true, the content is stored in the cache | ref.code
# return       $Content | page content or undef if the page has not been modified since
#                           the last request (given by $IfModifiedSince) | string
# return       $LastModified | time of last modification, i.e. when the content has been
#                           stored into the cache (given as epoch value) | int
#========================================================================================
sub compute {
    my $self = shift;
    my ($Object, $CacheKeyPart, $IfModifiedSince, $Sub) = @_;
    TestString('CacheKeyPart', $CacheKeyPart);

    my $PageCacheDuration = $Object->pageCacheDuration;

    my $LastModified = $self->_backend->lastModified($Object, $CacheKeyPart);
    my $IsUpToDate = defined $LastModified && ( time() - $LastModified ) / SECONDS_PER_MINUTE < $PageCacheDuration;

    my $Content;
    if( $IsUpToDate ) {
        if( defined $IfModifiedSince
         && defined $LastModified
         && $LastModified <= $IfModifiedSince
        ) {
            GetLog()->debug(sprintf 'not modified (%d <= %d): %s for object %s',
                                $LastModified, $IfModifiedSince, $CacheKeyPart, $Object->get('Path'));
            return undef, $LastModified;
        }

        GetLog()->debug(sprintf 'load from pagecache: %s for object %s', $CacheKeyPart, $Object->get('Path'));
        $Content = eval { $self->_backend->get($Object,  $CacheKeyPart) };
        Warning( GetError() ) if ExistsError();
    }

    if( !defined $Content ) {
        my $IsCacheable;
        ($Content, $IsCacheable) = $Sub->();
        if( $IsCacheable ) {
            GetLog()->debug(sprintf 'write to pagecache: %s for object %s', $CacheKeyPart, $Object->get('Path'));
            $self->_backend->set($Object, $CacheKeyPart, $Content);
        }
    }

    $LastModified = $self->_backend->lastModified($Object, $CacheKeyPart);

    return $Content, $LastModified;
}

#========================================================================================
# function     clearObjectCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageCache->clearObjectCache($Object);
#----------------------------------------------------------------------------------------
# description  Deletes all pages of a given object from the cache.
#----------------------------------------------------------------------------------------
# input        $Object | object | DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub clearObjectCache {
    my $self = shift;
    my ($Object) = @_;
    $self->_backend->clearObjectCache( $Object );
}

#========================================================================================
# function     clearSiteCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageCache->clearSiteCache;
#----------------------------------------------------------------------------------------
# description  Deletes all pages of a given site from the cache.
#----------------------------------------------------------------------------------------
# input        $Site | object | DE_EPAGES::Object::API::Object::Object
#========================================================================================
sub clearSiteCache {
    my $self = shift;
    my ($Site) = @_;
    $self->_backend->clearSiteCache( $self->_site );
}

#========================================================================================
# function     clearStoreCache
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PageCache->clearStoreCache;
#----------------------------------------------------------------------------------------
# description  Deletes all pages of the current database from the cache.
#========================================================================================
sub clearStoreCache {
    my $self = shift;
    $self->_backend->clearStoreCache;
}

#========================================================================================
# function     clearAllCaches
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PageCache->clearAllCaches;
#----------------------------------------------------------------------------------------
# description  Deletes all pages of all databases from the cache.
#========================================================================================
sub clearAllCaches() {
    my $self = shift;
    $self->_backend->clearAllCaches;
}

1;

