#========================================================================================
# package      DE_EPAGES::Object::API::PageCache::CompressedFile
# base         DE_EPAGES::Object::API::PageCache::File
# state        public
# description  Implements a compressed file-based page cache used by LZ4File, ZStdFile, ZipFile.
#========================================================================================
package DE_EPAGES::Object::API::PageCache::CompressedFile;
use base DE_EPAGES::Object::API::PageCache::File;

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw( ConvertPathChars GetFileContent  WriteFile MoveFile );
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::TestParameter qw( TestString TestObject );

sub new {
    my $class = shift;
    my (%Options) = @_;

    return $class->SUPER::new(%Options);
}

sub _compressedFileExtension {
    my $self = shift;
    die 'function _compressedFileExtension not defined in DE_EPAGES::Object::API::PageCache::CompressedFile';
}


sub _compressionType {
    my $self = shift;
    die 'function _compressionType not defined in DE_EPAGES::Object::API::PageCache::CompressedFile';
}


sub _fileName {
    my $self = shift;
    my ($Object, $CacheKeyPart) = @_;
    TestObject('Object', $Object);
    TestString('CacheKeyPart', $CacheKeyPart);

    return $self->_directoryName($Object).'/'.ConvertPathChars($CacheKeyPart).'.content' . $self->_compressedFileExtension; ## no critic qw(ProhibitPrivateSubroutineUse)
}

sub get {
    my $self = shift;
    my ($Object, $CacheKeyPart) = @_;

    my $FileName = $self->_fileName( $Object, $CacheKeyPart );
    return ${GetFileContent($FileName, { 'compressed' => $self->_compressionType, 'utf8' => 1 })};
}

sub set {
    my $self = shift;
    my ($Object, $CacheKeyPart, $Content) = @_;
    TestString('CacheKeyPart', $CacheKeyPart);
    Error('ContentUndefined', {
        'CacheKeyPart' => $CacheKeyPart,
        'Object' => $Object->pathString,
    }) unless defined $Content;

    my $FileName = $self->_fileName( $Object, $CacheKeyPart );
    WriteFile( $FileName.$$, \$Content, {
        'create_directory'=> 1,
        'utf8' => 1,
        'flock' => 1,
        'ignore_error' => 1,
        'compressed' => $self->_compressionType
    });
    eval {
        MoveFile($FileName . $$, $FileName);
    };
    if (ExistsError()) {
        LogError(GetError());
    }
    return;
}

1;

