#========================================================================================
# package      DE_EPAGES::PriceList::API::PriceList
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the main pricelist functions.
#========================================================================================
package DE_EPAGES::PriceList::API::PriceList;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetCustomerPriceLists
    GetProductPriceList
    IsTimeFrameDifferent
    GetUnityPriceList
    CreateUnityPriceListForCustomerGroup
    GetExportablePriceLists
    FillEmptyPriceListFields
);

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Customer::API::Shop qw( GetCustomerForUser );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadObject );
use DE_EPAGES::PriceList::API::Constants qw( UNITY_PRICELIST_PREFIX UNITY_PRICELIST_CUSTOMERGROUP_NAME );
use DE_EPAGES::Core::API::PerlTools qw( unique );
use DE_EPAGES::PriceList::API::Table::QuantityDiscount qw ( GetQuantityDiscountsByPriceListID );
use DE_EPAGES::PriceList::API::Table::ScalePrice qw ( GetScalePricesByPriceListID );

sub ErrorDefinition {
    return {
        'NoUnityShop' => 'Shop #Shop is not a Unity shop type and therefore has no Unity price list.',
    };
}

#========================================================================================
# function     GetCustomerPriceLists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aPriceLists = GetCustomerPriceLists( $Customer, $CurrencyID, $Shop, $User );
#----------------------------------------------------------------------------------------
# description  Returns the price list that is valid for the customer at the current time.
#               The price list may be assigned to the customer group or directly to the
#               customer.
#               Returns empty list if no price list is valid for this customer.
#----------------------------------------------------------------------------------------
# input        $Customer | Customer object | object
# input        $CurrencyID | currency id (eg. 'USD') | string
# input        $Shop | shop object (not used if customer is defined) | object
# input        $User | user object (not used if customer is defined)| object
# return       $aPriceLists | list of valid PriceList objects | ref.array.object
#========================================================================================
sub GetCustomerPriceLists($$$$) {
    my ($Customer, $CurrencyID, $Shop, $User) = @_;
    TestString( 'CurrencyID', $CurrencyID );

    if (not defined $Customer and
            defined $Shop and
            defined $User and not $User->get('IsAnonymous')) {
        $Customer = GetCustomerForUser($User, $Shop);
    }
    # try to find a pricelist via customer
    my @PriceLists = defined $Customer
        ? grep {$_->get('CurrencyID') eq $CurrencyID and $_->get('IsEffective')} @{$Customer->get('PriceLists')}
        : ();

    # try to find a pricelist via group
    my $CustomerGroup;
    if (defined $Customer) {
        $CustomerGroup = $Customer->get('CustomerGroup');
    } elsif (defined $User and not $User->get('IsAnonymous')) {
        $CustomerGroup = $Shop->get('NewMemberCustomerGroup');
    } else {
        $CustomerGroup = $Shop->get('NonMemberCustomerGroup');
    }
    if (defined $CustomerGroup) {
        push @PriceLists, grep {$_->get('CurrencyID') eq $CurrencyID and $_->get('IsEffective')} @{$CustomerGroup->get('PriceLists')};
    }
    return \@PriceLists;
}

#========================================================================================
# function     GetProductPriceList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PriceList = GetProductPriceList($Product, $Customer, $CurrencyID, $Shop, $User );
#----------------------------------------------------------------------------------------
# description  Returns the price list that is valid for the customer and product at the current time.
#               The price list may be assigned to the customer group or directly to the
#               customer.
#               Returns undef if no price list is valid for this customer.
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $Customer | customer object | object
# input        $CurrencyID | currency id (eg. 'USD') | string
# input        $Shop | shop object (not used if customer is defined) | object
# input        $User | user object (not used if customer is defined)| object
# return       $PriceList | valid PriceList objects | object
#========================================================================================
sub GetProductPriceList($$$$$) {
    my ($Product, $Customer, $CurrencyID, $Shop, $User) = @_;
    foreach my $PriceList (@{GetCustomerPriceLists($Customer, $CurrencyID, $Shop, $User)}) {
        return $PriceList
            if scalar @{$PriceList->scalePrices($Product)}
            or scalar @{$PriceList->quantityDiscounts($Product)}
            or scalar @{$PriceList->valueDiscounts($Product)};
    }

    # do the next steps only for unity
    return undef unless $Shop->get('IsUnityRegistered');

    my $PriceListAlias = sprintf('%s%s', UNITY_PRICELIST_PREFIX, $CurrencyID);
    if ($Shop->folder('PriceLists')->existsChild($PriceListAlias)) {
        return $Shop->folder('PriceLists')->child($PriceListAlias);
    }

    return undef;
}

#========================================================================================
# function     IsTimeFrameDifferent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NoOverlap = IsTimeFrameDifferent($hTimeSpan1, $hTimeSpan2)
#----------------------------------------------------------------------------------------
# description  Returns true if 2 time spans do not overlap.
#----------------------------------------------------------------------------------------
# input        $hTimeSpan1 | time span with hash keys ValidFrom, ValidTo | ref.hash
# input        $hTimeSpan2 | time span with hash keys ValidFrom, ValidTo | ref.hash
# return       $NoOverlap | true if time spans do not overlap | boolean
#========================================================================================
sub IsTimeFrameDifferent {
    my ( $A,  $B) = @_;
    if (  ( $A->{'ValidFrom'} <= $B->{'ValidFrom'}    and  $B->{'ValidFrom'} <= $A->{'ValidTo'} ) or
          ( $A->{'ValidFrom'} <= $B->{'ValidTo'}      and  $B->{'ValidTo'}   <= $A->{'ValidTo'} ) or
          ( $B->{'ValidFrom'} <= $A->{'ValidFrom'}    and  $A->{'ValidFrom'} <= $B->{'ValidTo'} ) or
          ( $B->{'ValidFrom'} <= $A->{'ValidTo'}      and  $A->{'ValidTo'}   <= $B->{'ValidTo'} )
       ) {
        return 0; #there is an overlapping
    } else {
        return 1; # there is not any overlapping
    }
}

#========================================================================================
# function     GetUnityPriceList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PriceList = GetUnityPriceList($Shop, $CreateIfNotExists );
#----------------------------------------------------------------------------------------
# description  Returns the current active price list of a now shop
#               Creates a new price list if it not exists and $CreateIfNotExists is true.
#               Returns undef if no price list exists and $CreateIfNotExists is not true.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $CreateIfNotExists | create shop pricelist if not exists (default 0)(optional) | integer
# return       $PriceList | PriceList objects | object
#========================================================================================
sub GetUnityPriceList($;$) {
    my ($Shop, $CreateIfNotExists) = @_;

    if ($Shop->get('IsUnityRegistered')) {
        my $PriceList;
        my $PriceListAlias = _getUnityPriceListAlias($Shop);
        if ($Shop->folder('PriceLists')->existsChild($PriceListAlias)) {
            $PriceList = $Shop->folder('PriceLists')->child($PriceListAlias);
        } elsif ($CreateIfNotExists) {
            my $CustomerGroup = _getOrCreateUnityCustomerGroup($Shop);
            $PriceList = _createUnityPriceList($Shop);
            $PriceList->addCustomerGroup($CustomerGroup);
        }
        return $PriceList;
    } else {
        Error('NoUnityShop', { 'Shop' => $Shop->pathString });
    }
}

#========================================================================================
# function     CreateUnityPriceListForCustomerGroup
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PriceList = CreateUnityPriceListForCustomerGroup($CustomerGroup);
#----------------------------------------------------------------------------------------
# description  Returns the ID for a Pricelist with the same alias as the CustomerGroup.
#               Creates new PriceList if the there is no PriceList with the CustomerGroup
#               alias.
#----------------------------------------------------------------------------------------
# input        $CustomerGroup | CustomerGroup object | object
# return       $PriceList | PriceList object | object
#========================================================================================
sub CreateUnityPriceListForCustomerGroup($) {
    my ($CustomerGroup) = @_;

    my $Shop    = $CustomerGroup->getSite();
    my $Folder = $Shop->folder('PriceLists', { 'Position' => 30 });
    my $PriceListAlias = $CustomerGroup->alias;
    my $PriceList;

    if ($Folder->existsChild($PriceListAlias)) {
        $PriceList =  $Folder->child($PriceListAlias);
    } else {
        my $Class = LoadClassByAlias('PriceList');
        my $hPriceList = {
            'Parent'     => $Folder,
            'Alias'      => $PriceListAlias,
            'ValidFrom'  => DateTime->now(),
            'ValidTo'    => DateTime->new('year' => 2999, 'month' => 1, 'day' => 1),
            'CurrencyID' => $Shop->get('CurrencyID'),
        };
        $PriceList = $Class->insertObject($hPriceList);

        # assign unity customergroup to pricelist
        $PriceList->addCustomerGroup($CustomerGroup);

    }
    return $PriceList;
}

#========================================================================================
# function     GetExportablePriceLists
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetExportablePriceLists($PriceList);
#----------------------------------------------------------------------------------------
# description  Returns a true value if the price list is allowed to be eported or not.
#               1 for yes, 0 for no.
#----------------------------------------------------------------------------------------
# input        $Shop         | shop object   | object
#----------------------------------------------------------------------------------------
# return       $aExportablePriceLists  | exportable pricelists | arrayref
#========================================================================================
sub GetExportablePriceLists {
    my ($Shop) = @_;

    my $aExportablePriceLists=[];

    # return empty arrayref if  PriceList feature is not enabled
    return $aExportablePriceLists unless $Shop->featureMaxValue('PriceLists');

    foreach my $PriceList (@{$Shop->get('PriceLists')}) {

        # Do not export if not currently effective, other CurrencyID then Shop CurrencyID or have no customergroups assigned
        next if (!$PriceList->get('IsEffective'));
        next if ($PriceList->get('CurrencyID') ne $Shop->get('CurrencyID'));
        my $aPriceListCustomerGroups = $PriceList->get('CustomerGroups');
        next unless (scalar(@{$aPriceListCustomerGroups}));

        push @$aExportablePriceLists, $PriceList;
    }
    return $aExportablePriceLists;
}

#========================================================================================
# function     FillEmptyPriceListFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       FillEmptyPriceListFields($Shop);
#----------------------------------------------------------------------------------------
# description  Fill possible empty fields for Scaleprices, Quantitydiscounts and
#               Globalvaluediscounts.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub FillEmptyPriceListFields {
    my ($Shop) = @_;
    my $hInfoShop = $Shop->get([qw(TaxModel TaxMatrix PriceLists)]);

    my $ahShopPriceLists = $hInfoShop->{'PriceLists'};
    my $ShopTaxModel = $hInfoShop->{'TaxModel'};
    my $ShopTaxMatrix = $hInfoShop->{'TaxMatrix'};

    foreach my $PriceList (@$ahShopPriceLists) {
        my $PriceListCurrencyID = $PriceList->get('CurrencyID');
        my @ExistingProductID; #store ProductIDs for products with scaleprices and/or quantitydiscounts

        #get all scaleprices for current pricelist, already sorted by quantity
        my $ScalePricesForPriceList = GetScalePricesByPriceListID($Shop->id, $PriceList->id);
        my @ItemsScalePrices;
        foreach my $ScalePriceEntry (@$ScalePricesForPriceList) {
            #save ProductID for later use
            push @ExistingProductID, $ScalePriceEntry->{'ProductID'};

            #get TaxClass for Product
            my $TaxClass = LoadObject($ScalePriceEntry->{'ProductID'})->getWithParent('TaxClass');
            push (@ItemsScalePrices, {
                'ProductID'   => $ScalePriceEntry->{'ProductID'},
                'PriceListID' => $PriceList->id,
                'Quantity'    => $ScalePriceEntry->{'Quantity'},
                #we need gross prices, DB saves net prices
                'Price' => $ShopTaxMatrix->net2gross($PriceListCurrencyID, $ScalePriceEntry->{'Price'},
                    {   'TaxModel' => $ShopTaxModel,
                        'TaxClass' => $TaxClass
                    }, 0)
            });
        }

        #get all quantitydiscounts for current pricelist, already sorted by quantity
        my $QuantityDiscountsForPriceList = GetQuantityDiscountsByPriceListID($Shop->id, $PriceList->id);
        my @ItemsQuantityDiscount; #array to store quantity discounts

        foreach my $QuantityDiscountEntry (@$QuantityDiscountsForPriceList) {
            #save ProductID for later use
            push @ExistingProductID, $QuantityDiscountEntry->{'ProductID'};

            push (@ItemsQuantityDiscount, {
                'ProductID'     => $QuantityDiscountEntry->{'ProductID'},
                'PriceListID'   => $PriceList->id,
                'Quantity'      => $QuantityDiscountEntry->{'Quantity'},
                'Discount'      => $QuantityDiscountEntry->{'Discount'},
            });
        }

        foreach my $ProductID (unique @ExistingProductID) {
            my $Product = LoadObject($ProductID);
            my $ProductListPrice = $Product->getListPrice($PriceListCurrencyID);
            my $ahScalePrices = [];
            my $ahQuantityDiscounts = [];
            my $aQuantities = [];

            my @ahPricesDiscounts;

            #get all quantities from scaleprices and quantitydiscounts for the current product
            foreach my $ScalePriceQuantity (@ItemsScalePrices) {
                next unless $ScalePriceQuantity->{ProductID} eq $ProductID;
                push @$aQuantities, $ScalePriceQuantity->{'Quantity'};
            }

            foreach my $QuantityDiscountQuantity (@ItemsQuantityDiscount) {
                next unless $QuantityDiscountQuantity->{ProductID} eq $ProductID;
                push @$aQuantities, $QuantityDiscountQuantity->{'Quantity'};
            }

            foreach my $Quantity (unique @$aQuantities) {
                #get scaleprice/quantitydiscount for current product per quantity
                my ($ScalePricePrice) = grep { $_->{'Quantity'} eq $Quantity && $_->{'ProductID'} eq $ProductID } @ItemsScalePrices;
                my ($QuantityDiscountValue) = grep { $_->{'Quantity'} eq $Quantity && $_->{'ProductID'} eq $ProductID } @ItemsQuantityDiscount;

                push(@ahPricesDiscounts, {
                    'PriceListID' => $PriceList->id,
                    'ProductID'   => $Product->id,
                    'Price'       => $ScalePricePrice->{'Price'},
                    'Quantity'    => $Quantity,
                    'Discount'    => $QuantityDiscountValue->{'Discount'}
                });
            }

            #we need to sort by quantity
            # {
            #     'PriceListID' => 24490,
            #     'Price' => '666',
            #     'ProductID' => '23828',
            #     'Discount' => '22',
            #     'Quantity' => '2.0000'
            # },
            # {
            #     'PriceListID' => 24490,
            #     'Price' => '44',
            #     'ProductID' => '23828',
            #     'Discount' => '6',
            #     'Quantity' => '4.0000'
            # },
            # {
            #     'PriceListID' => 24490,
            #     'Price' => '666',
            #     'ProductID' => '23828',
            #     'Discount' => '6',
            #     'Quantity' => '6.0000'
            # }
            @ahPricesDiscounts = sort { $a->{Quantity} <=> $b->{Quantity} } @ahPricesDiscounts;

            if (scalar(@ahPricesDiscounts)) {
                for my $i (0..scalar(@ahPricesDiscounts)-1) {
                    if (defined $ahPricesDiscounts[$i]->{'Price'}) {
                        push @$ahScalePrices, {
                            'Quantity'  => $ahPricesDiscounts[$i]->{'Quantity'},
                            'Price'     => $ahPricesDiscounts[$i]->{'Price'},
                        };
                    }
                    else {
                        if ($i ne '0') {
                            for (my $j = $i; $j >= 1; $j--) {
                                if (defined $ahPricesDiscounts[$j-1]->{'Price'}) {
                                    push @$ahScalePrices, {
                                        'Quantity'  => $ahPricesDiscounts[$i]->{'Quantity'},
                                        'Price'     => $ahPricesDiscounts[$j-1]->{'Price'},
                                    };
                                    #update current element to be faster for the next element
                                    $ahPricesDiscounts[$i]->{'Price'} = $ahPricesDiscounts[$j-1]->{'Price'};
                                    last;
                                }
                            }
                        } else {
                            push @$ahScalePrices, {
                                'Quantity'  => $ahPricesDiscounts[$i]->{'Quantity'},
                                'Price'     => $ProductListPrice,
                            };
                            #update current element to be faster for the next element
                            $ahPricesDiscounts[$i]->{'Price'} = $ProductListPrice;
                        }
                    }

                    #do not add/update quantitydiscounts for the bulkprices prislist
                    next if $PriceList->alias eq UNITY_PRICELIST_PREFIX.$PriceListCurrencyID;
                    if (defined $ahPricesDiscounts[$i]->{'Discount'}) {
                        push @$ahQuantityDiscounts, {
                            'Quantity'  => $ahPricesDiscounts[$i]->{'Quantity'},
                            'Discount'     => $ahPricesDiscounts[$i]->{'Discount'},
                        };
                    }
                    else {
                        if ($i ne '0') {
                            for (my $j = $i; $j >= 1; $j--) {
                                if (defined $ahPricesDiscounts[$j-1]->{'Discount'}) {
                                    push @$ahQuantityDiscounts, {
                                        'Quantity'  => $ahPricesDiscounts[$i]->{'Quantity'},
                                        'Discount'     => $ahPricesDiscounts[$j-1]->{'Discount'},
                                    };
                                    #update current element to be faster for the next element
                                    $ahPricesDiscounts[$i]->{'Discount'} = $ahPricesDiscounts[$j-1]->{'Discount'};
                                    last;
                                }
                            }
                        } else {
                            push @$ahQuantityDiscounts, {
                                'Quantity'  => $ahPricesDiscounts[$i]->{'Quantity'},
                                'Discount'     => '0',
                            };
                            #update current element to be faster for the next element
                            $ahPricesDiscounts[$i]->{'Discount'} = '0';
                        }
                    }
                }
            }

            $PriceList->setScalePrices($Product, $ShopTaxModel, $ahScalePrices);
            $PriceList->setQuantityDiscounts($Product, $ShopTaxModel, $ahQuantityDiscounts);
        }
        #do not add/update GlobalValueDiscounts for the bulkprices prislist
        next if $PriceList->alias eq UNITY_PRICELIST_PREFIX.$PriceListCurrencyID;

        # fill empty fields for globalvaluediscounts
        my $ahPriceListGlobalValueDiscounts = $PriceList->get('GlobalValueDiscounts');
        # Example:
        # 'Attributes' => {
        #  'PriceListID' => 24491,
        #      'DiscountAbsolute' => undef,
        #      'Value' => '50',
        #      'ClassID' => 11912,
        #      'Discount' => 10,
        #      'GlobalValueDiscountID' => 10
        #  }

    next unless scalar @$ahPriceListGlobalValueDiscounts;

        for my $i (0..scalar(@$ahPriceListGlobalValueDiscounts)-1) {
            if ($i == 0) {
                if (!(defined @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'DiscountAbsolute'})) {
                    @{$ahPriceListGlobalValueDiscounts}[$i]->set({
                        'Discount'          => @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'Discount'},
                        'DiscountAbsolute'  =>'0'
                    });
                    #update current element to be faster for the next element
                    @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'DiscountAbsolute'} = '0';
                } elsif (!(defined @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'Discount'})) {
                    @{$ahPriceListGlobalValueDiscounts}[$i]->set({
                        'Discount'          => '0',
                        'DiscountAbsolute'  => @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'DiscountAbsolute'}
                    });
                    #update current element to be faster for the next element
                    @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'Discount'} = '0;'
                }
            } else {
                for (my $j = $i; $j >= 1; $j--) {
                    if (!(defined @{$ahPriceListGlobalValueDiscounts}[$j]->{'Attributes'}->{'DiscountAbsolute'})) {
                        @{$ahPriceListGlobalValueDiscounts}[$i]->set({
                            'Discount'          => @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'Discount'},
                            'DiscountAbsolute'  => @{$ahPriceListGlobalValueDiscounts}[$j-1]->{'Attributes'}->{'DiscountAbsolute'}
                        });
                        #update current element to be faster for the next element
                        @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'DiscountAbsolute'} = @{$ahPriceListGlobalValueDiscounts}[$j-1]->{'Attributes'}->{'DiscountAbsolute'};
                        last;
                    } elsif (!(defined @{$ahPriceListGlobalValueDiscounts}[$j]->{'Attributes'}->{'Discount'})) {
                        @{$ahPriceListGlobalValueDiscounts}[$i]->set({
                            'Discount'          => @{$ahPriceListGlobalValueDiscounts}[$j-1]->{'Attributes'}->{'Discount'},,
                            'DiscountAbsolute'  => @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'DiscountAbsolute'}
                        });
                        #update current element to be faster for the next element
                        @{$ahPriceListGlobalValueDiscounts}[$i]->{'Attributes'}->{'Discount'} = @{$ahPriceListGlobalValueDiscounts}[$j-1]->{'Attributes'}->{'Discount'};
                        last;
                    }
                }
            }
        }
    }
}

sub _getUnityPriceListAlias {
    my ($Shop) = @_;
    return sprintf('%s%s', UNITY_PRICELIST_PREFIX, $Shop->get('CurrencyID'));
}

# creates a fixed pricelist for unity shops
sub _createUnityPriceList {
    my ($Shop) = @_;

    my $Folder  = $Shop->folder('PriceLists', { 'Position'=> 30 });
    my $PriceListAlias = _getUnityPriceListAlias($Shop);
    my $PriceList;
    if ($Folder->existsChild($PriceListAlias)) {
        $PriceList = $Folder->child($PriceListAlias);
    } else {
        GetLog->debug("Adding new Pricelist $PriceListAlias");
        my $Class = LoadClassByAlias('PriceList');
        my $hPriceList = {
            'Parent'     => $Folder,
            'Alias'      => $PriceListAlias,
            'ValidFrom'  => DateTime->now(),
            'ValidTo'    => DateTime->new('year'=>2999, 'month'=>1,'day'=>1),
            'CurrencyID' => $Shop->get('CurrencyID'),
        };
        $PriceList = $Class->insertObject($hPriceList);
    }
    return $PriceList;
}

# creates a fixed customergroup for unity shops
sub _getOrCreateUnityCustomerGroup {
    my ($Shop) = @_;

    my $CustomerGroups = $Shop->get('CustomerGroups');
    my $CustomerGroup;
    if (scalar @$CustomerGroups) {
        $CustomerGroup = $CustomerGroups->[0];
    } else {
        GetLog->debug("adding CustomerGroup ".UNITY_PRICELIST_CUSTOMERGROUP_NAME);
        my $Folder  = $Shop->folder('Groups', { 'Position'=> 20 });
        my $Class = LoadClassByAlias('CustomerGroup');
        my $hCustomerGroup = {
            'Parent'              => $Folder,
            'Alias'               => UNITY_PRICELIST_CUSTOMERGROUP_NAME,
            'IsNewMemberDefault'  => 1,
            'IsNonMemberDefault'  => 1,
        };
        $CustomerGroup = $Class->insertObject($hCustomerGroup);
        GetLog->debug(sprintf('Assigning Customer to CustomerGroup %s', UNITY_PRICELIST_CUSTOMERGROUP_NAME));
        # skip Customers without Users as they cannot add to the basket again
        for (@{$Shop->get('Customers')}) {
            next unless scalar @{$_->get('Users')};
            $_->set( {'CustomerGroup' => $CustomerGroup} );
        }
    }
    return $CustomerGroup;
}

1;
