#========================================================================================
# package      DE_EPAGES::Zettle::API::ProductChangeHelper
# state        public
#----------------------------------------------------------------------------------------
# description  class that implements methods related to handling product changes
#========================================================================================
package DE_EPAGES::Zettle::API::ProductChangeHelper;
use base qw ( Exporter );

use strict;

use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Zettle::API::Constants qw ( IMAGE_LARGE_PROFILE ZETTLE_ATTRIBUTES_PROFILE );

our @EXPORT_OK = qw (
    GetDeletedProducts
    GetModifiedImageProducts
    GetModifiedBasicDataProducts
    GetModifiedPriceProducts
    GetModifiedProductsByProfile
);

#========================================================================================
# function     GetDeletedProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductsDeleted = GetDeletedProducts($Shop, $DeletedAfter)
#----------------------------------------------------------------------------------------
# description  Retrieve the GUIDs of products deleted after $DeletedAfter.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $DeletedAfter | reference date for finding deleted products | Datetime 
#----------------------------------------------------------------------------------------
# return       $aProductsDeleted | list of GUIDs of deleted products | ref.array.string
#========================================================================================
sub GetDeletedProducts {
    my ($Shop, $DeletedAfter) = @_;

    my $Iterator = $Shop->getDeletedObjects('Product', $DeletedAfter);

    my @aProductsDeleted = ();
    while(my $hDeleted = $Iterator->next){
        my $ProductGuid = $hDeleted->{'GUID'};
        push(@aProductsDeleted, $ProductGuid);
    }

    return \@aProductsDeleted;
}

#========================================================================================
# function     GetModifiedImageProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aModifiedImageProducts = GetModifiedImageProducts($Shop, $ModifiedAfter)
#----------------------------------------------------------------------------------------
# description  Retrieve products whose image was modified after $ModifiedAfter.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $ModifiedAfter | reference date for finding modified product images | Datetime 
#----------------------------------------------------------------------------------------
# return       $aModifiedImageProducts | list of products | ref.array.object
#========================================================================================
sub GetModifiedImageProducts {
    my ($Shop, $ModifiedAfter) = @_;

    my $aModifiedImageProducts = GetModifiedProductsByProfile($Shop, $ModifiedAfter, IMAGE_LARGE_PROFILE);

    return $aModifiedImageProducts;
}

#========================================================================================
# function     GetModifiedBasicDataProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aModifiedProducts = GetModifiedBasicDataProducts($Shop, $ModifiedAfter)
#----------------------------------------------------------------------------------------
# description  Retrieve products whose Zettle-related attributes were modified after $ModifiedAfter.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $ModifiedAfter | reference date for finding modified products | Datetime 
#----------------------------------------------------------------------------------------
# return       $aModifiedProducts | list of products | ref.array.object
#========================================================================================
sub GetModifiedBasicDataProducts {
    my ($Shop, $ModifiedAfter) = @_;

    my $aModifiedProducts = GetModifiedProductsByProfile($Shop, $ModifiedAfter, ZETTLE_ATTRIBUTES_PROFILE);

    return $aModifiedProducts;
}

#========================================================================================
# function     GetModifiedPriceProducts
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aModifiedProducts = GetModifiedPriceProducts($Shop, $ModifiedAfter)
#----------------------------------------------------------------------------------------
# description  Retrieve products whose price (PriceList) was modified after $ModifiedAfter.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $ModifiedAfter | reference date for finding modified product prices | Datetime 
#----------------------------------------------------------------------------------------
# return       $aModifiedProducts | list of products | ref.array.object
#========================================================================================
sub GetModifiedPriceProducts {
    my ($Shop, $ModifiedAfter) = @_;

    my $aModifiedProducts = GetModifiedProductsByProfile($Shop, $ModifiedAfter, 'ListPrice');

    return $aModifiedProducts;
}

#========================================================================================
# function     GetModifiedProductsByProfile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aModifiedProducts = GetModifiedProductsByProfile($Shop, $LastUpdate, $ProfileName)
#----------------------------------------------------------------------------------------
# description  Retrieve products updated after $LastUpdate with regards to given profile.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $LastUpdate | reference date for finding modified product prices | Datetime 
# input        $ProfileName | name of a Product LastUpdateProfile | string
#----------------------------------------------------------------------------------------
# return       $aModifiedProducts | list of products | ref.array.object
#========================================================================================
sub GetModifiedProductsByProfile {
    my ($Shop, $LastUpdate, $ProfileName) = @_;

    my $Profile = LoadClassByAlias('Product')->lastUpdateProfile($ProfileName);

    # Load shop objects updated after given date relative to $Profile
    my $Iterator = $Shop->getUpdatedObjects($Profile, $LastUpdate);
        
    my @aModifiedProducts;
    while( my $Item = <$Iterator> ) {
        my $Object = $Item->{'Object'};
        push(@aModifiedProducts, $Object) if ref $Object eq 'DE_EPAGES::Product::API::Object::Product';
    }

    return \@aModifiedProducts;
}

1;
