#========================================================================================
# package      DE_EPAGES::Zettle::API::Table::ZettleDelete
# state        public
# description  interface for table ZettleDelete
#========================================================================================
package DE_EPAGES::Zettle::API::Table::ZettleDelete;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    InsertZettleDelete
    UpdateZettleDelete
    DeleteZettleDelete
    GetAllPKeyZettleDeletes
    GetAllInfoZettleDeletes
    GetInfoZettleDelete
    ExistsZettleDelete
    GetPKeyZettleDeletesBySiteID
    GetInfoZettleDeletesBySiteID
    GetPKeyZettleDeletesByClassID
    GetInfoZettleDeletesByClassID
);

use DE_EPAGES::Core::API::Error         qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestInteger TestFloat TestDateTime ); ## no critic qw(RequireCorrectImports)
use DE_EPAGES::Trigger::API::Trigger    qw ( TriggerHook );
use DE_EPAGES::Zettle::DAL::Zettle;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults_ZettleDelete' => 'ZettleDelete doesn\'t exist (GUID=#GUID)',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     InsertZettleDelete
#----------------------------------------------------------------------------------------
# syntax       InsertZettleDelete($hZettleDelete);
#----------------------------------------------------------------------------------------
# description  Creates an entry of ZettleDelete.
#----------------------------------------------------------------------------------------
# input        $hZettleDelete | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.hash
#========================================================================================
sub InsertZettleDelete ($) {
    my ($hZettleDelete) = @_;
    DAL_InsertZettleDelete($hZettleDelete);
    my $GUID = $hZettleDelete->{'GUID'};
    TriggerHook('API_InsertZettleDelete', { 'GUID' => $GUID });
    return;
}

#========================================================================================
# function     UpdateZettleDelete
#----------------------------------------------------------------------------------------
# syntax       UpdateZettleDelete($GUID, $hZettleDelete);
# syntax       UpdateZettleDelete($hZettleDelete); # if $hZettleDelete includes GUID
#----------------------------------------------------------------------------------------
# description  Updates a ZettleDelete if any element was changed.
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
# input        $hZettleDelete | with following keys:
#               <ul>
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.hash
#========================================================================================
sub UpdateZettleDelete ($;$) {
    my ($GUID, $hZettleDelete) = @_;
    if (ref $GUID eq 'HASH') {
        $hZettleDelete = $GUID;
        $GUID = $hZettleDelete->{'GUID'};
    } else {
        $hZettleDelete->{'GUID'} = $GUID;
    }
    TestString('GUID',$GUID);
    TriggerHook('API_BeforeUpdateZettleDelete', $hZettleDelete );
    DAL_UpdateZettleDelete($GUID, $hZettleDelete);
    TriggerHook('API_AfterUpdateZettleDelete', $hZettleDelete );
    return 1;
}

#========================================================================================
# function     DeleteZettleDelete
#----------------------------------------------------------------------------------------
# syntax       DeleteZettleDelete($GUID);
#----------------------------------------------------------------------------------------
# description  Deletes a ZettleDelete.
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key - varchar(36)) | string
#========================================================================================
sub DeleteZettleDelete ($) {
    my ($GUID) = @_;
    if (ref $GUID eq 'HASH') {
        my $hInfo = $GUID;
        $GUID = $hInfo->{'GUID'};
    }
    TestString('GUID',$GUID);
    TriggerHook('API_DeleteZettleDelete', { 'GUID' => $GUID } );
    DAL_DeleteZettleDelete($GUID);
    return;
}

#========================================================================================
# function     GetAllPKeyZettleDeletes
#----------------------------------------------------------------------------------------
# syntax       $aZettleDeleteIDs = GetAllPKeyZettleDeletes();
#----------------------------------------------------------------------------------------
# description  Returns all ZettleDelete identifiers.
#----------------------------------------------------------------------------------------
# return       $aZettleDeleteIDs | GUID of the deleted object (varchar(36)) | ref.array.string
#========================================================================================
sub GetAllPKeyZettleDeletes () { DAL_GetAllPKeyZettleDeletes(); }

#========================================================================================
# function     GetAllInfoZettleDeletes
#----------------------------------------------------------------------------------------
# syntax       $ahZettleDeletes = GetAllInfoZettleDeletes();
#----------------------------------------------------------------------------------------
# description  Returns all objects of ZettleDelete.
#----------------------------------------------------------------------------------------
# return       $ahZettleDeletes | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| ref.array.hash
#========================================================================================
sub GetAllInfoZettleDeletes () { DAL_GetAllInfoZettleDeletes(); }

#========================================================================================
# function     GetInfoZettleDelete
#----------------------------------------------------------------------------------------
# syntax       $hZettleDelete = GetInfoZettleDelete($GUID);
#----------------------------------------------------------------------------------------
# description  Returns the ZettleDelete by identifier.
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
#----------------------------------------------------------------------------------------
# return       $hZettleDelete | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| ref.hash
#========================================================================================
sub GetInfoZettleDelete ($) {
    my ($GUID) = @_;
    TestString('GUID',$GUID);
    my $hZettleDelete = DAL_GetInfoZettleDelete($GUID);
    Error('NoResults_ZettleDelete', { 'GUID' => $GUID }) unless defined $hZettleDelete;
    return $hZettleDelete;
}

#========================================================================================
# function     ExistsZettleDelete
#----------------------------------------------------------------------------------------
# syntax       $exists = ExistsZettleDelete($GUID);
#----------------------------------------------------------------------------------------
# description  Returns true if the ZettleDelete exists.
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key - varchar(36)) | string
#----------------------------------------------------------------------------------------
# return       $exists | 1/0  the ZettleDelete exists (yes/no) | boolean
#========================================================================================
sub ExistsZettleDelete ($) {
    my ($GUID) = @_;
    TestString('GUID',$GUID);
    my $hZettleDelete = DAL_GetInfoZettleDelete($GUID);
    return defined $hZettleDelete ? 1 : 0;
}

#========================================================================================
# function     GetPKeyZettleDeletesBySiteID
#----------------------------------------------------------------------------------------
# syntax       $aKeys = GetPKeyZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of ZettleDelete by alternate keys SiteID and ClassID and DeletedOn.
#----------------------------------------------------------------------------------------
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". (optional) | int
# input        $DeletedOn | Date and time of the deletion on an object (optional) | datetime
#----------------------------------------------------------------------------------------
# return       $aKeys | GUID of the deleted object (primary key - varchar(36)) | ref.array.string
#========================================================================================
sub GetPKeyZettleDeletesBySiteID ($;$$) {
    my ($SiteID, $ClassID, $DeletedOn) = @_;
    TestInteger('SiteID',$SiteID);
    TestInteger('ClassID',$ClassID) if defined $ClassID;
    TestDateTime('DeletedOn',$DeletedOn) if defined $DeletedOn;
    return DAL_GetPKeyZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
}

#========================================================================================
# function     GetInfoZettleDeletesBySiteID
#----------------------------------------------------------------------------------------
# syntax       $ahZettleDeletes = GetInfoZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
#----------------------------------------------------------------------------------------
# description  Returns objects of ZettleDelete by alternate keys SiteID and ClassID and DeletedOn.
#----------------------------------------------------------------------------------------
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". (optional) | int
# input        $DeletedOn | Date and time of the deletion on an object (optional) | datetime
#----------------------------------------------------------------------------------------
# return       $ahZettleDeletes | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.array.hash
#========================================================================================
sub GetInfoZettleDeletesBySiteID ($;$$) {
    my ($SiteID, $ClassID, $DeletedOn) = @_;
    TestInteger('SiteID',$SiteID);
    TestInteger('ClassID',$ClassID) if defined $ClassID;
    TestDateTime('DeletedOn',$DeletedOn) if defined $DeletedOn;
    return DAL_GetInfoZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
}

#========================================================================================
# function     GetPKeyZettleDeletesByClassID
#----------------------------------------------------------------------------------------
# syntax       $aKeys = GetPKeyZettleDeletesByClassID($ClassID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of ZettleDelete by alternate key ClassID.
#----------------------------------------------------------------------------------------
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# return       $aKeys | GUID of the deleted object (primary key - varchar(36)) | ref.array.string
#========================================================================================
sub GetPKeyZettleDeletesByClassID ($) {
    my ($ClassID) = @_;
    TestInteger('ClassID',$ClassID);
    return DAL_GetPKeyZettleDeletesByClassID($ClassID);
}

#========================================================================================
# function     GetInfoZettleDeletesByClassID
#----------------------------------------------------------------------------------------
# syntax       $ahZettleDeletes = GetInfoZettleDeletesByClassID($ClassID);
#----------------------------------------------------------------------------------------
# description  Returns objects of ZettleDelete by alternate key ClassID.
#----------------------------------------------------------------------------------------
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# return       $ahZettleDeletes | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.array.hash
#========================================================================================
sub GetInfoZettleDeletesByClassID ($) {
    my ($ClassID) = @_;
    TestInteger('ClassID',$ClassID);
    return DAL_GetInfoZettleDeletesByClassID($ClassID);
}

#========================================================================================
# function     OnDeleteClass
#----------------------------------------------------------------------------------------
# syntax       OnDeleteClass($hParams);
#----------------------------------------------------------------------------------------
# description  Deletes ZettleDelete of this model when a Class is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of object Class, key is
#               <ul>
#                   <li>ClassID
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteClass {
    my ($hParams) = @_;
    DeleteZettleDelete($_) foreach @{GetPKeyZettleDeletesByClassID($hParams->{'ClassID'})};
}

#========================================================================================
# function     OnDeleteSite
#----------------------------------------------------------------------------------------
# syntax       OnDeleteSite($hParams);
#----------------------------------------------------------------------------------------
# description  Deletes ZettleDelete of this model when a(n) Site is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of object Site, key is
#               <ul>
#                   <li>SiteID
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteSite {
    my ($hParams) = @_;
    DeleteZettleDelete($_) foreach @{GetPKeyZettleDeletesBySiteID($hParams->{'SiteID'})};
}

1;
