#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateDTO
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle payload of incoming stock level update messages (Zettle webhooks triggered on InventoryBalanceChanged).
#               See <DE_EPAGES::Zettle::API::WebService:DTO::WebhookEventDTO>.
#               See https://github.com/iZettle/api-documentation/blob/master/pusher-api/user-guides/subscriptions.md#understand-how-events-work
#----------------------------------------------------------------------------------------
# notes        The information is structured in the following way:
#               <ul>
#                   <li> organizationUuid - organization uuid -  string
#                   <li> externalUuid - uuid used to discard self-generated updates - string
#                   <li> updated - who performed the update and when it occured:
#                       <ul>
#                           <li> userType - type of user - string
#                           <li> timestamp - event timestamp in ISO-8601 format - string
#                           <li> clientUuid - #TODO: uuid of user who made the update??? - string
#                           <li> uuid - #TODO: product owner uuid??? - string
#                       </ul>
#                   <li> balanceBefore - list of variation stock levels before update. Array of objects with the following information:
#                       <ul>
#                           <li> created - product creation timestamp in ISO-8601 format - string
#                           <li> organizationUuid - organization uuid -  string
#                           <li> locationUuid - location uuid where product is stored - string
#                           <li> productUuid - super product uuid - string
#                           <li> variantUuid - variation uuid - string
#                           <li> balance - stock level (string representation of integer value) - string
#                       </ul>
#                   <li> balanceAfter - list of variation stock levels after update. Array of objects with the following information:
#                       <ul>
#                           <li> created - product creation timestamp in ISO-8601 format - string
#                           <li> organizationUuid - organization uuid -  string
#                           <li> locationUuid - location uuid where product is stored - string
#                           <li> productUuid - super product uuid - string
#                           <li> variantUuid - variation uuid - string
#                           <li> balance - stock level (string representation of integer value) - string
#                       </ul>
#               </ul>
#----------------------------------------------------------------------------------------
# example      {
#                   "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
#                   "externalUuid": undef,
#                   "updated": {
#                                   "userType": "USER",
#                                   "timestamp": "2021-08-24T17:10:44.856+0000",
#                                   "clientUuid": "51e57b0d-1ea2-48a4-b0a0-91d90424e62c",
#                                   "uuid": "1a5431c6-824d-11eb-92c1-8c85ccaff8db"
#                              },
#                   "balanceAfter": [
#                           {
#                               "created": "2021-08-24T17:10:44.866+0000",
#                               "balance": "2",
#                               "productUuid": "611ea330-d0fd-114f-2671-b976395ffb2b",
#                               "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
#                               "locationUuid": "8f59b9ee-828b-11eb-b512-f9a20a1c5cc2",
#                               "variantUuid": "611ea330-684f-1b83-2d32-b976395ffbd0"
#                           },
#                           {
#                               "created": "2021-08-24T17:10:44.868+0000",
#                               "balance": "5",
#                               "productUuid": "611ea330-d0fd-114f-2671-b976395ffb2b",
#                               "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
#                               "locationUuid": "8f59b9ee-828b-11eb-b512-f9a20a1c5cc2",
#                               "variantUuid": "611ea330-3adc-1137-2727-b976395ffb45"
#                           }
#                   ],
#                   "balanceBefore": [
#                           {
#                               "created": "2021-08-23T10:54:03.124+0000",
#                               "balance": "2",
#                               "productUuid": "611ea330-d0fd-114f-2671-b976395ffb2b",
#                               "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
#                               "locationUuid": "8f59b9ee-828b-11eb-b512-f9a20a1c5cc2",
#                               "variantUuid": "611ea330-684f-1b83-2d32-b976395ffbd0"
#                           },
#                           {
#                               "created": "2021-08-23T10:54:03.125+0000",
#                               "balance": "3",
#                               "productUuid": "611ea330-d0fd-114f-2671-b976395ffb2b",
#                               "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
#                               "locationUuid": "8f59b9ee-828b-11eb-b512-f9a20a1c5cc2",
#                               "variantUuid": "611ea330-3adc-1137-2727-b976395ffb45"
#                           }
#                   ]
#               }
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateDTO;

use strict;

use JSON qw ( to_json );

use DE_EPAGES::Zettle::API::WebService::DTO::EventInitiatorDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateBalanceDTO;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateDTO->new();
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateDTO
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

# input        $OrganizationUuid | Organization uuid | string
sub organizationUuid {
    my $self = shift;
    my ($OrganizationUuid) = @_;
    return $self->{'organizationUuid'} unless defined($OrganizationUuid);
    $self->{'organizationUuid'} = $OrganizationUuid;
}

# input        $ExternalUuid | External identifier for the update used to avoid processing our own stock updates.
#                               We fill this field with MerchantUUID in the stock updates we generate | string
sub externalUuid {
    my $self = shift;
    my ($ExternalUuid) = @_;
    return $self->{'externalUuid'} unless defined($ExternalUuid);
    $self->{'externalUuid'} = $ExternalUuid;
}

# input        $Initiator | Information on who performed the update and when it occured.
#                            See <DE_EPAGES::Zettle::API::WebService::DTO::EventInitiatorDTO> | object
sub initiator {
    my $self = shift;
    my ($Initiator) = @_;
    return $self->{'initiator'} unless defined($Initiator);
    $self->{'initiator'} = $Initiator;
}

# input        $aBalanceBefore | list of variation stock levels before update. Array of StockUpdateBalanceDTO objects.
#                                 See <DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateBalanceDTO> | ref.array.object
sub balanceBefore {
    my $self = shift;
    my ($BalanceBefore) = @_;
    return $self->{'balanceBefore'} unless defined($BalanceBefore);
    $self->{'balanceBefore'} = $BalanceBefore;
}

# input        $aBalanceAfter | list of variation stock levels after update. Array of StockUpdateBalanceDTO objects.
#                                See <DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateBalanceDTO> | ref.array.object
sub balanceAfter {
    my $self = shift;
    my ($BalanceAfter) = @_;
    return $self->{'balanceAfter'} unless defined($BalanceAfter);
    $self->{'balanceAfter'} = $BalanceAfter;
}

#========================================================================================
# function     fromHash
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StockUpdateDTO = DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateDTO->fromHash($hStockUpdate);
#----------------------------------------------------------------------------------------
# description  parse JSON contents from Pusher API subscription response to produce a StockUpdateDTO object
#----------------------------------------------------------------------------------------
# input        $hStockUpdate | Hash parsed from JSON structure retrieved from incoming stock update triggered by a webhook event.
#                               See notes and example sections in class documentation for structure details | ref.hash
#----------------------------------------------------------------------------------------
# return       $StockUpdateDTO | a subscription response object | DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateDTO
#========================================================================================
sub fromHash {
    my $class = shift;
    my ($hStockUpdate) = @_;

    my $StockUpdateDTO = $class->new();

    $StockUpdateDTO->organizationUuid($hStockUpdate->{'organizationUuid'});
    $StockUpdateDTO->externalUuid($hStockUpdate->{'externalUuid'});

    my $EventInitiatorDTO = DE_EPAGES::Zettle::API::WebService::DTO::EventInitiatorDTO->fromHash($hStockUpdate->{'updated'});
    $StockUpdateDTO->initiator($EventInitiatorDTO);

    my @aBalanceBeforeDTO = ();
    foreach my $hbalanceBefore (@{$hStockUpdate->{'balanceBefore'}}){
        my $BalanceBeforeDTO = DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateBalanceDTO->fromHash($hbalanceBefore);
        push(@aBalanceBeforeDTO, $BalanceBeforeDTO);
    }
    $StockUpdateDTO->balanceBefore(\@aBalanceBeforeDTO);
    
    my @aBalanceAfterDTO = ();
    foreach my $hbalanceAfter (@{$hStockUpdate->{'balanceAfter'}}){
        my $BalanceAfterDTO = DE_EPAGES::Zettle::API::WebService::DTO::StockUpdateBalanceDTO->fromHash($hbalanceAfter);
        push(@aBalanceAfterDTO, $BalanceAfterDTO);
    }
    $StockUpdateDTO->balanceAfter(\@aBalanceAfterDTO);    

    return $StockUpdateDTO;
}

sub TO_JSON {
    return { %{ shift() } };
}

sub toJson {
    my $self = shift;

    return to_json($self, { 'pretty' => 1, 'allow_blessed' => 1, 'convert_blessed' => 1 });
}

1;
