#========================================================================================
# package      DE_EPAGES::Zettle::DAL::SQL::Zettle
# state        private
# description  Generic SQL database interface for tables:
#               <ul>
#                   <li>zettledelete
#               </ul>
#========================================================================================
package DE_EPAGES::Zettle::DAL::SQL::Zettle;

use strict;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );

## no critic qw(ProhibitLongLines)

my %TABLE_DEF = (
    'ZettleDelete' => {
        'GUID' => ['guid','String'],
        'SiteID' => ['siteid','Integer'],
        'ClassID' => ['classid','Integer'],
        'PathFromSite' => ['pathfromsite','String'],
        'ZettleUUID' => ['zettleuuid','String'],
        'DeletedOn' => ['deletedon','DateTime'],
    },
);

sub ErrorDefinition {
    return {
        'TableNotExists'  => 'table #Table does not exist at package DE_EPAGES::Zettle::DAL::SQL::Zettle',
        'ColumnNotExists' => 'column #Column does not exist at table #Table at package DE_EPAGES::Zettle::DAL::SQL::Zettle',
    };
}

sub TableDef {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[0];
}

sub TableType {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[1];
}

#========================================================================================
# function     SetZettleDeleted
#----------------------------------------------------------------------------------------
# syntax       $Package->SetZettleDeleted($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Writes an object into the zettledeleted table with the current time.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                  <li>SiteID        - Site ID              - int
#                  <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                  <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                  <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#               </ul> | ref.hash.*
#========================================================================================
sub SetZettleDeleted {
    my $self = shift;
    my ($dbi, $hParameter) = @_;

    my %Params = %$hParameter; # copy hash
    $Params{'DeletedOn'} = GetCurrentDateTime();
    if( defined GetInfoZettleDelete($Params{GUID}) ) {
        my $GUID = delete $Params{GUID};
        UpdateZettleDelete($dbi, $GUID, \%Params);
    }
    else {
        InsertZettleDelete($dbi, \%Params);
    }
}

#========================================================================================
# function     GetDeletedObjects
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Package->GetDeletedObjects($dbi, $SiteID, $ClassID, $DeletedAfter);
#----------------------------------------------------------------------------------------
# description  Returns deleted objects of a given class from a site.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# input        $DeletedAfter | Date and time of the deletion on an object | datetime
#----------------------------------------------------------------------------------------
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| Iterator of hashes
#========================================================================================
sub GetDeletedObjects {
    my $self = shift;
    my ($dbi, $SiteID, $ClassID, $DeletedAfter) = @_;

    my $sql = <<'END_SQL';
SELECT guid, pathfromsite, zettleuuid, deletedon
FROM zettledelete
WHERE siteid = ?
  AND classid = ?
  AND deletedon >= ?
END_SQL
    return $dbi->iterator($sql, [$SiteID, $ClassID, $DeletedAfter],
        ['String', 'String', 'String', 'DateTime'],
        [qw(GUID PathFromSite ZettleUUID DeletedOn)]
    );
}

#========================================================================================
# function     ClearDeletedObjects
#----------------------------------------------------------------------------------------
# syntax       $Package->ClearDeletedObjects($dbi, $SiteID, $ClassID, $DeletedBefore);
#----------------------------------------------------------------------------------------
# description  Deletes information about deleted objects in the site of a given type.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# input        $DeletedBefore | Date and time of the deletion on an object | datetime
#========================================================================================
sub ClearDeletedObjects {
    my $self = shift;
    my ($dbi, $SiteID, $ClassID, $DeletedBefore) = @_;

    my $sql = <<'END_SQL';
DELETE FROM zettledelete
WHERE siteid = ?
  AND classid = ?
  AND deletedon < ?
END_SQL
    return $dbi->do($sql, [$SiteID, $ClassID, $DeletedBefore]);
}

#========================================================================================
# function     InsertZettleDelete
#----------------------------------------------------------------------------------------
# syntax       $Package->InsertZettleDelete($dbi, $hParameter);
#----------------------------------------------------------------------------------------
# description  Inserts an entry into table zettledelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $hParameter | with following keys:
#               <ul>
#                  <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                  <li>SiteID        - Site ID              - int
#                  <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                  <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                  <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                  <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.hash.*
#========================================================================================
sub InsertZettleDelete {
    my $self = shift;
    my ($dbi, $hParameter) = @_;
    $dbi->do('INSERT INTO zettledelete (guid, siteid, classid, pathfromsite, zettleuuid, deletedon) VALUES (?, ?, ?, ?, ?, ?)',
        [ @{$hParameter}{qw( GUID SiteID ClassID PathFromSite ZettleUUID DeletedOn )} ]
    );
}

#========================================================================================
# function     UpdateZettleDelete
#----------------------------------------------------------------------------------------
# syntax       UpdateZettleDelete($dbi, $GUID, $hParameter);
#----------------------------------------------------------------------------------------
# description  Updates an entry of table zettledelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
# input        $hParameter | with following keys:
#               <ul>
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| ref.hash.*
#========================================================================================
sub UpdateZettleDelete {
    my $self = shift;
    my ($dbi, $GUID, $hParameter) = @_;
    if (scalar keys %$hParameter) {
        my $sql = 'UPDATE zettledelete SET ';
        $sql .= join(', ', map { $self->TableDef('ZettleDelete',$_).'=?' } sort keys %$hParameter);
        $sql .= ' WHERE guid=?';
        $dbi->do($sql, [(map { $hParameter->{$_} } sort keys %$hParameter), $GUID]);
    }
    return;
}

#========================================================================================
# function     DeleteZettleDelete
#----------------------------------------------------------------------------------------
# syntax       $Package->DeleteZettleDelete($dbi, $GUID);
#----------------------------------------------------------------------------------------
# description  Removes an entry of table objectdelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
#========================================================================================
sub DeleteZettleDelete {
    my $self = shift;
    my ($dbi, $GUID) = @_;
    $dbi->do('DELETE FROM zettledelete WHERE guid=?', [$GUID]);
    return;
}

#========================================================================================
# function     GetAllPKeyZettleDeletes
#----------------------------------------------------------------------------------------
# syntax       $aZettleDeleteIDs = $Package->GetAllPKeyZettleDeletes($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table zettledelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $aZettleDeleteIDs | GUID of the deleted object | reference to array of varchar(36)
#========================================================================================
sub GetAllPKeyZettleDeletes {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT guid FROM zettledelete', [], ['String']);
    return [ map { $_->[0] } @$dbResult ];
}

#========================================================================================
# function     GetAllInfoZettleDeletes
#----------------------------------------------------------------------------------------
# syntax       $ahZettleDeleteIDs = $Package->GetAllInfoZettleDeletes($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table zettledelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahZettleDeleteIDs | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoZettleDeletes {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT guid, siteid, classid, pathfromsite, zettleuuid, deletedon FROM zettledelete', [],
        ['String', 'Integer', 'Integer', 'String', 'String', 'DateTime'],
        [qw(GUID SiteID ClassID PathFromSite ZettleUUID DeletedOn)]
    );
}

#========================================================================================
# function     GetInfoZettleDelete
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoZettleDelete($dbi, $GUID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
# return       $hResult | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| ref.hash.*
#========================================================================================
sub GetInfoZettleDelete {
    my $self = shift;
    my ($dbi, $GUID) = @_;
    return $dbi->execute('SELECT guid, siteid, classid, pathfromsite, zettleuuid, deletedon FROM zettledelete WHERE guid=?',
        [$GUID],
        ['String', 'Integer', 'Integer', 'String', 'String', 'DateTime'],
        [qw(GUID SiteID ClassID PathFromSite ZettleUUID DeletedOn)]
    )->[0];
}

#========================================================================================
# function     GetPKeyZettleDeletesBySiteID
#----------------------------------------------------------------------------------------
# syntax       $aResult = $Package->GetPKeyZettleDeletesBySiteID($dbi, $SiteID, $ClassID, $DeletedOn);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table zettledelete by index i_zettledelete.
#               Uses the index i_zettledelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". (optional) | int
# input        $DeletedOn | Date and time of the deletion on an object (optional) | datetime
#----------------------------------------------------------------------------------------
# return       $aGUIDs | GUID of the deleted object (primary key - varchar(36)) | ref.array.string
#========================================================================================
sub GetPKeyZettleDeletesBySiteID {
    my $self = shift;
    my ($dbi, $SiteID, $ClassID, $DeletedOn) = @_;
    my (@WhereClauses, @Parameter);
    if (defined $DeletedOn) {
        unshift @WhereClauses, 'deletedon = ?';
        unshift @Parameter, $DeletedOn;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    if (defined $ClassID) {
        unshift @WhereClauses, 'classid = ?';
        unshift @Parameter, $ClassID;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    unshift @WhereClauses, 'siteid = ?';
    unshift @Parameter, $SiteID;

    my $sql = 'SELECT guid FROM zettledelete WHERE ' . join(' AND ', @WhereClauses);
    my $dbResult = $dbi->execute($sql, \@Parameter, ['String']);
    return [ map { $_->[0] } @$dbResult ];
}

#========================================================================================
# function     GetInfoZettleDeletesBySiteID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoZettleDeletesBySiteID($dbi, $SiteID, $ClassID, $DeletedOn);
#----------------------------------------------------------------------------------------
# description  get full row of table zettledelete by index i_zettledelete
#               Uses the index i_zettledelete.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". (optional) | int
# input        $DeletedOn | Date and time of the deletion on an object (optional) | datetime
#----------------------------------------------------------------------------------------
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| ref.array.hash
#========================================================================================
sub GetInfoZettleDeletesBySiteID {
    my $self = shift;
    my ($dbi, $SiteID, $ClassID, $DeletedOn) = @_;
    my (@WhereClauses, @Parameter);
    if (defined $DeletedOn) {
        unshift @WhereClauses, 'deletedon = ?';
        unshift @Parameter, $DeletedOn;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    if (defined $ClassID) {
        unshift @WhereClauses, 'classid = ?';
        unshift @Parameter, $ClassID;
    }
    else {
        @WhereClauses = ();
        @Parameter = ()
    }
    unshift @WhereClauses, 'siteid = ?';
    unshift @Parameter, $SiteID;

    my $sql = 'SELECT guid, siteid, classid, pathfromsite, zettleuuid, deletedon FROM zettledelete WHERE ' . join(' AND ', @WhereClauses);
    return $dbi->execute($sql, \@Parameter,
        ['String', 'Integer', 'Integer', 'String', 'String', 'DateTime'],
        [qw(GUID SiteID ClassID PathFromSite ZettleUUID DeletedOn)]
    );
}

#========================================================================================
# function     GetPKeyZettleDeletesByClassID
#----------------------------------------------------------------------------------------
# syntax       $aResult = $Package->GetPKeyZettleDeletesByClassID($dbi, $ClassID);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table zettledelete by index i_zettledelete_class.
#               Uses the index i_zettledelete_class.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# return       $aGUIDs | GUID of the deleted object (primary key) | reference to array of varchar(36)
#========================================================================================
sub GetPKeyZettleDeletesByClassID {
    my $self = shift;
    my ($dbi, $ClassID) = @_;
    my @Parameter = ($ClassID);
    my $sql = 'SELECT guid FROM zettledelete WHERE classid = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['String']);
    return [ map { $_->[0] } @$dbResult ];
}

#========================================================================================
# function     GetInfoZettleDeletesByClassID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoZettleDeletesByClassID($dbi, $ClassID);
#----------------------------------------------------------------------------------------
# description  get full row of table zettledelete by index i_zettledelete_class
#               Uses the index i_zettledelete_class.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoZettleDeletesByClassID {
    my $self = shift;
    my ($dbi, $ClassID) = @_;
    return $dbi->execute('SELECT guid, siteid, classid, pathfromsite, zettleuuid, deletedon FROM zettledelete WHERE classid = ?',
        [$ClassID],
        ['String', 'Integer', 'Integer', 'String', 'String', 'DateTime'],
        [qw(GUID SiteID ClassID PathFromSite ZettleUUID DeletedOn)]
    );
}

1;
