use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( ExistsClassByAlias LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::Zettle::API::Constants qw ( ZETTLE_ENVIRONMENT_CLASS ZETTLE_OBJECT_ALIAS );
use DE_EPAGES::Zettle::API::ZettleService;

# Store to be used if 'storename' argument is not present
use constant DEFAULT_STORE => 'Store';

my $ShopAlias;

# Run product exports and/or cleanup operations for all ZettleEnvironments.
sub Main {

    local $| = 1;

    my ($Help, $NoOutput, $Password, $StoreName, $ExportUpdates, $CleanupEnvironments, $Force);

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
        'export'      => \$ExportUpdates,
        'cleanup'     => \$CleanupEnvironments,
        'force'       => \$Force,
    );

    $StoreName //= DEFAULT_STORE;

    usage() if $Help;
    usage() unless $ExportUpdates || $CleanupEnvironments;

    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            # make sure that the cartridge is installed
            unless(ExistsCartridgeByPackage('DE_EPAGES::Zettle')) {
                print "Store $StoreName: Cartridge Zettle is not installed.\n" unless $NoOutput;
                return;
            }

            my @EnvironmentsForExport = ();
            my @EnvironmentsForCleanup = ();
            # Run script on a specific shop if 'shop' argument is present (and different from 'all')
            if(defined $ShopAlias && $ShopAlias ne 'all'){
                my $Shop;
                eval {
                    $Shop = LoadObjectByPath('/Shops')->child( $ShopAlias );
                };
                if(ExistsError()){
                    printf STDERR ("[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias);
                    return;
                }

                return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

                my $ZettleEnvironment = $Shop->child(ZETTLE_OBJECT_ALIAS);
                my $RequiresCleanup = $ZettleEnvironment->requiresCleanup();

                # Compile the list of ZettleEnvironments where cleanup operations need to be executed.
                if($CleanupEnvironments){                    
                    push (@EnvironmentsForCleanup, $ZettleEnvironment) if $RequiresCleanup || $Force;
                }

                # Compile the list of ZettleEnvironments where export operations need to be executed.
                if($ExportUpdates){
                    push (@EnvironmentsForExport, $ZettleEnvironment) if (! $RequiresCleanup && $ZettleEnvironment->isScheduledExportPending()) || $Force;
                }
            } else {
                unless(ExistsClassByAlias(ZETTLE_ENVIRONMENT_CLASS)){
                    printf STDERR ("[%s] WARN: Class '%s' does not exist.\n", ZETTLE_ENVIRONMENT_CLASS);
                    return;
                }

                # Retrieve all store Zettle instances
                my $aZettleEnvironments = LoadClassByAlias(ZETTLE_ENVIRONMENT_CLASS)->objects;
                
                foreach my $ZettleEnvironment ( @{ $aZettleEnvironments }){
                    if($ZettleEnvironment->requiresCleanup()){
                        # Compile the list of ZettleEnvironments where cleanup operations need to be executed.
                        push (@EnvironmentsForCleanup, $ZettleEnvironment) if $CleanupEnvironments;
                    }else{
                        # Compile the list of ZettleEnvironments where export operations need to be executed.
                        push (@EnvironmentsForExport, $ZettleEnvironment) if($ExportUpdates && $ZettleEnvironment->isScheduledExportPending());
                    }
                }
            }

            # Cleanup operations
            RunCleanupJob(\@EnvironmentsForCleanup, $NoOutput, $StoreName);

            # Export product changes (creation, deletion, updates) to Zettle.
            RunExportJob(\@EnvironmentsForExport, $NoOutput, $StoreName);
        },
    );

    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $timemsg = sprintf( 'need %.3f seconds', $seconds );
    print $timemsg. "\n" unless $NoOutput;
    GetLog()->debug("Zettle maintenance operations on Store '$StoreName': $timemsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

# Perform cleanup operations for given ZettleEnvironments
sub RunCleanupJob {    ## no critic (ProhibitFunctionSize)
    my ($Environments, $NoOutput, $StoreName) = @_;
    
    return unless scalar @{$Environments};
    
    foreach my $Environment ( @{$Environments} ) {
        my $EnvPath = $Environment->pathString;
        my $Shop = $Environment->getSite();
        
        # NOTE: cleanup will be executed even when Zettle feature is disabled to make sure we don't leave any orphan environment behind

        my $aWarnings = BlockWarnings(
            sub {
                eval{
                    # Clean up Zettle environment
                    printf STDOUT ("[%s] Cleaning up Zettle environment for shop: %s\n",
                                    $StoreName, $Shop->alias) unless $NoOutput;
                    # Delete Zettle object
                    $Environment->delete();
                };

                if(ExistsError()){
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $EnvPath });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning ( @{$aWarnings} ) {
            LogError($Warning);
            # ignore failed logins in scheduler calls (still in the logs).
            next if ($Warning->code eq 'WebserviceError' && $NoOutput);

            printf STDERR ( "[%s] WARN: %s: %s\n",
                $StoreName,
                $Warning->vars->{Path}//'path unknown' ,
                substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }

    return;
}

# Export product create, update and delete
sub RunExportJob {    ## no critic (ProhibitFunctionSize)
    my ($Environments, $NoOutput, $StoreName) = @_;
    
    return unless scalar @{$Environments};

    my $Feature = LoadFeatureByAlias('Zettle');
    foreach my $Environment ( @{$Environments} ) {
        my $EnvPath = $Environment->pathString;
        my $Shop = $Environment->getSite();

        # skip export if Zettle feature is disabled for this shop
        my $FeatureValue = $Shop->featureMaxValue($Feature);
        next if ( not $FeatureValue );

        # skip if scheduled product export is disabled for this environment
        # don't skip if user requested this specific shopalias.
        next if ($Environment->exportScheduleInterval() == 0
                    && !(defined $ShopAlias && $Shop->alias eq $ShopAlias));

        # skip if shop is ClosedByProvider
        next if ($Shop->get('ClosedByProvider'));

        # skip if product export is running
        next if ($Environment->get('ProductIsExporting'));

        my $aWarnings = BlockWarnings(
            sub {
                eval{
                    # Send product updates to Zettle
                    printf STDOUT ("[%s] Exporting product updates to Zettle for shop: %s\n",
                                    $StoreName, $Shop->alias) unless $NoOutput;
                    $Environment->exportProductUpdates();
                };
                if(ExistsError()){
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $EnvPath });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning ( @{$aWarnings} ) {
            LogError($Warning);

            printf STDERR ("[%s] WARN: %s: %s\n",
                            $StoreName,
                            $Warning->vars->{Path}//'path unknown' ,
                            substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }

    return;
}

sub usage {
    print <<END_USAGE;
Description:
    Clean up disconnected Zettle environments and export product updates to Zettle.

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Default: Store)
    -shop         shop alias             (optional)   (Default: all)
flags:
    -nooutput   do not print any output
    -help       show the command line options
    -export     export product updates to Zettle (includes product creation, deletion and updates)
    -cleanup    run cleanup operations for Zettle environments set as 'MarkedForDeletion'
    -force      force task execution even if Environment doesn't match conditions (only applies when used with the 'shop' parameter)

Example:
    perl $0 -shop DemoShop -export -force
    perl $0 -storename Store -export -cleanup
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
